@@ -61,6 +61,26 @@ FILE	*mfp = NULL;
 extern char *postoffice; /* At mail_open() */
 
 
+#define RFC821_822QUOTE(newcp,cp) \
+	if (cp && strchr(cp,'\\') != NULL && *cp != '"') {	\
+	  char *s1 = cp, *s2;					\
+	  /* For this we can add at most 2 new quote chars */	\
+	  s2 = newcp = emalloc(strlen(cp)+4);			\
+	  *s2++ = '"';						\
+	  while (*s1) {						\
+	    if (*s1 == '@')					\
+	      break; /* Unquoted AT --> move to plain copying! */ \
+	    if (*s1 == '\\' && s1[1] != 0)			\
+	      *s2++ = *s1++;					\
+	    /* Normal copy */					\
+	    *s2++ = *s1++;					\
+	  }							\
+	  *s2++ = '"';						\
+	  while (*s1)						\
+	    *s2++ = *s1++;					\
+	  cp = newcp;						\
+	}
+
 int
 main(argc, argv)
 	int argc;
@@ -83,6 +103,7 @@ main(argc, argv)
 	char	*notificationopt = NULL;
 	char	*returnopt = NULL;
 	char	*envidstr = NULL;
+	char	*newcp = NULL;
 
 	if ((cp = strrchr(argv[0], '/')) != NULL)
 		progname = ++cp;
@@ -462,9 +483,16 @@ otherprog:
 		  fprintf(mfp, "bodytype %s\n", bodytype);
 		if (fullname != NULL && *fullname != '\0')
 		  /* maybe we should put it in the environment? */
+
 		  fprintf(mfp, "fullname %s\n", fullname);
+
+		RFC821_822QUOTE(newcp,from);
+
 		if (from != NULL && *from != '\0')
 		  fprintf(mfp, "from %s\n", from);
+
+		if (newcp) free(newcp); newcp = NULL;
+
 		if (envidstr)
 		  fprintf(mfp, "envid %s\n", envidstr);
 		if (returnopt)
@@ -656,6 +684,7 @@ doabort()
 	exit(EX_TEMPFAIL);
 }
 
+
 /* haa@cs.hut.fi:  Sometimes address components contain pure junk.. */
 void
 check_and_print_to(mfp, addr, notify)
@@ -663,7 +692,7 @@ check_and_print_to(mfp, addr, notify)
 	char *addr, *notify;
 {
 	char *copy = NULL, *printme = addr, *from, *to;
-	char *s;
+	char *s, newcp = NULL;
 
 	if (addr == NULL) {
 	  fprintf(stderr, "sendmail: Argument botch: NULL address\n");
@@ -681,6 +710,9 @@ check_and_print_to(mfp, addr, notify)
 
 	  printme = copy;
 	}
+
+	RFC821_822QUOTE(newcp,printme);
+
 	/* FIRST 'todsn', THEN 'to' -header! */
 	fprintf(mfp, "todsn ORCPT=rfc822;");
 	s = printme;
@@ -697,6 +729,7 @@ check_and_print_to(mfp, addr, notify)
 	putc('\n',mfp);
 	fprintf(mfp, "to %s\n", printme);
 	if (copy) free(copy);
+	if (newcp) free(newcp);
 }
 
 #if 0
