@@ -50,8 +50,10 @@ struct rcpt {
 					   Thus enabling rewrite of the
 					   header on the transport. */
 	char		***newmsgheadercvt; /* the rewrite results */
-	long		id;		/* the index of this address */
-	long		lockoffset;	/* the index of the address lock */
+	int		id;		/* the index of this address */
+	int		lockoffset;	/* the index of the address lock */
+	int		headeroffset;
+	int		drptoffset;
 	int		status;		/* current delivery sysexit code */
 	struct ctldesc	*desc;		/* backpointer to descriptor */
 	/* XX: something needed for XOR address lists */
@@ -63,6 +65,7 @@ struct ctldesc {
 	char		*verbose;	/* file for verbose logging */
 	char		*envid;		/* DSN ENVID data */
 	char		*dsnretmode;	/* DSN RET=-mode */
+	time_t		msgmtime;	/* Message file arrival time */
 	long		msgbodyoffset;	/* offset of message body in msgfile */
 	long		msgsizeestimate; /* Estimate of the msg size */
 	int		msgfd;		/* message file I/O descriptor */
@@ -115,17 +118,17 @@ struct mimestate {
 #ifndef __
 # ifndef __STDC__
 #  define __(x) ()
-#  ifdef __GNUC__
-#   define const    __const
-#   define volatile __volatile
-#  else
-#   define const
-#   define volatile
-#  endif
 # else
 #  define __(x) x
 # endif
 #endif
+#ifdef __GNUC__
+#   define const    __const
+#   define volatile __volatile
+#else
+#   define const
+#   define volatile
+#endif
 
 /* ctlopen.c: */
 extern void            ctlfree __((struct ctldesc *dp, void *anyp));
@@ -139,7 +142,7 @@ extern char           *notaryacct __((int rc, char *okstr));
 		/* NOTARY: addres / action / status / diagnostic  */
 extern void 	       notaryreport __((const char*,const char*,const char*,const char*));
 #if defined(HAVE_STDARG_H)
-extern void	       diagnostic __(( struct rcpt *, int, int, char *, ... ));
+extern void	       diagnostic __((struct rcpt *rp, int rc, int timeout, char *fmt, ... ));
 #else
 extern void	       diagnostic __((/* struct rcpt *, int, int, char *,... */));
 #endif
@@ -152,7 +155,8 @@ extern int	getrrtype __((char *, int, int, int, FILE *));
 extern int	       emptyline __((char *line, int size));
 
 /* esyslib.c: */
-extern int cistrcmp __((char *s1, char *s2));
+extern int cistrcmp  __((const char *s1, const char *s2));
+extern int cistrncmp __((const char *s1, const char *s2, const int n));
 extern int efstat __((int fd, struct stat *stbuf));
 extern int emkdir __((char *dirpath, int mode));
 extern int eopen __((char *s, int f, int m));
@@ -220,9 +224,16 @@ extern RETSIGTYPE wantout __(());
 /* warning.c */
 extern int warning __(());
 
-/* skip821address.c */
+/* lib/skip821address.c */
 extern char *skip821address __((char *s));
 
+/* lib/taspoolid.c */
+extern void taspoolid __((char *buf, int buflen, time_t mtime, char *path));
+
+/* tasyslog.c */
+extern void tatimestr __((char *buf, int dt));
+extern void tasyslog __((struct rcpt *rp, int xdelay, char *wtthost, char *stat, char *msg));
+
 #ifndef CISTREQ
 #define  CISTREQ(x, y)      (cistrcmp((char *)(x), (char *)(y)) == 0)
 #define  CISTREQN(x, y, n)  (cistrncmp((char *)(x), (char *)(y), n) == 0)
