@@ -10,12 +10,12 @@
 
 /* scanner tables: definition of character classes according to RFC822 */
 
-#define _h	01	/* header field */
-#define _w	02	/* linear white-space character (space / htab) */
-#define _d	04	/* digit */
-#define _c	010	/* control */
-#define _a	020	/* alphabetic */
-#define _l	040	/* line feed */
+#define _h	  01	/* header field */
+#define _w	  02	/* linear white-space character (space / htab) */
+#define _d	  04	/* digit */
+#define _c	 010	/* control */
+#define _a	 020	/* alphabetic */
+#define _l	 040	/* line feed */
 #define _r	0100	/* carriage return */
 #define _s	0200	/* specials */
 #define _8      0400	/* 8th bit on -- illegal on Headers! */
@@ -73,28 +73,53 @@ _8,	_8,	_8,	_8,	_8,	_8,	_8,	_8	/* 370 - 377 */
  */
 
 int
-hdr_status(cp, lbuf, n)
+hdr_status(cp, lbuf, n, octo)
 	register u_char	*cp, *lbuf;
-	int	n;
+	int	n, octo;
 {
 	u_char *cpin = cp;
 
 	if (*cp == ' ' || *cp == '\t') {
-		while ((cp < lbuf + n) && (rfc_ctype[*cp] & (_w|_l)))
-			++cp;
-		if (cp == lbuf + n)
-			/* a line containing only whitespace is EOH */
-			return -1;
-		/* a continuation line (folded header) */
-		return 0;
+	  while ((cp < lbuf + n) && (rfc_ctype[*cp] & (_w|_l)))
+	    ++cp;
+	  if (cp == lbuf + n)
+	    /* a line containing only whitespace is EOH */
+	    return -1;
+	  /* a continuation line (folded header) */
+	  return 0;
+	}
+	if (!octo) {
+	  while ((cp < lbuf + n) && (rfc_ctype[*cp] & _h))
+	    ++cp;
+	  if ((cp < lbuf + n) &&
+	      (*cp == ':') /*&& (cp > cpin)*/)	/* header line */
+	    return cp - lbuf;
+	  /* if we get to here, we have a malformed header line */
+	  /* if (*cp == ':' && cp == cpin) return -1; */
+	  return lbuf - cp;
+	} else {
+	  /* complex calling relations -- we are parsing alias database,
+	     and we want to have spaces allowed in the left-hand side.. */
+	  char quote = 0;
+	  while (cp < lbuf + n) {
+	    char c = *cp;
+	    if (c == '\\') {
+	      ++cp;
+	      if (cp >= (lbuf + n))
+		break;
+	    }
+	    if (c == quote)
+	      quote = 0;
+	    else if (c == '"')
+	      quote = '"';
+	    else if (!quote && !(rfc_ctype[c] & _h))
+	      break;
+	    ++cp;
+	  }
+	  if (cp < lbuf + n && *cp == ':')
+	    return cp - lbuf;
+	  return lbuf - cp;
 	}
-	while ((cp < lbuf + n) && (rfc_ctype[*cp] & _h))
-		++cp;
-	if ((cp < lbuf + n) && (*cp == ':') /*&& (cp > cpin)*/)	/* header line */
-		return cp - lbuf;
-	/* if we get to here, we have a malformed header line */
-	/* if (*cp == ':' && cp == cpin) return -1; */
-	return lbuf - cp;
 }
 
 #define MKERROR(msg,prev)	tn = makeToken((u_char *)(msg), strlen(msg)); \
