@@ -44,7 +44,7 @@ static char *mmap_filename;
 #ifdef HAVE_MMAP
 /* Sun gets size_t wrong, and these follow, thanks to my #defines! */
 #undef caddr_t
-#undef size_t
+/* #undef size_t */
 #undef u_char
 #include <sys/types.h>
 #include <sys/mman.h>
@@ -102,7 +102,7 @@ size_t nbytes;
 		return (univptr_t) -1;
 	mmf.i_end = mmf.i_data + mmf.i_size;
 	mmf.i_alloced = nbytes;
-#if	defined(MADV_RANDOM) && !defined(NO_MADVICE)
+#if	defined(MADV_RANDOM) && !defined(NO_MADVISE)
 	/* Advise vm system of random access pattern */
 	(void) madvise(mmf.i_data, mmf.i_size, MADV_RANDOM);
 #endif
