@@ -235,7 +235,7 @@ again:
 		if (recur)	/* we're at the end of the road of a |**| */
 			bp = kleene;
 		else {
-			*swp = stash(cwd, pwd-cwd, *swp);
+			*swp = stash(cwd, (int)(pwd-cwd), *swp);
 			return 1;
 		}
 	}
@@ -299,14 +299,14 @@ again:
 				if (*start != 0 && start != kleene)
 					continue;
 				++count;
-				*swp = stash(cwd, (pwd+namlen)-cwd, *swp);
+				*swp = stash(cwd, (int)((pwd+namlen)-cwd), *swp);
 			}
 		}
 		if (*eoname == 0) {	/* end of the globbable name */
 			/* we aren't interested in descending directories */
 			if (!recur || glob_match(start, eoname, dp->d_name))
 				++count,
-				*swp = stash(cwd, (pwd+namlen)-cwd, *swp);
+				*swp = stash(cwd, (int)((pwd+namlen)-cwd), *swp);
 		} else if (!recur) {
 			/* we are only interested in directories */
 			i = glut(cwd, pwd+namlen+1, eoname, recur, swp);
