@@ -196,10 +196,11 @@ reg_exec (prog, str)
 		return (0);
 	}
 
+#define DEBUG /* allow traceing here as happens with tregexp.c ! */
 #ifdef  DEBUG
 	if (D_compare) {
 		fprintf(stderr,
-			"%*scomparing %s and ", 4*funclevel, " ", prog->pattern);
+			"%*sscomparing '%s' and ", 4*funclevel, " ", prog->pattern);
 		if (str != NULL)
 			fprintf(stderr, "'%s'\n", str);
 		else
@@ -230,7 +231,7 @@ reg_exec (prog, str)
 #ifdef  DEBUG
 	if (D_matched) {
 		fprintf(stderr,
-			"%*smatched %s and ", 4*funclevel, " ", prog->pattern);
+			"%*ssmatched '%s' and ", 4*funclevel, " ", prog->pattern);
 		if (str != NULL)
 			fprintf(stderr, "'%s'\n", str);
 		else
