@@ -175,11 +175,13 @@ vfprintf(fp, fmt, ap)
 
 int
 #ifdef HAVE_STDARG_H
-siofprintf(FILE *fp, char *fmt
-# ifdef __STDC__
-	   , ...
-# endif
-	   )
+#ifdef __STDC__
+siofprintf(FILE *fp, char *fmt, ...)
+#else /* Non ANSI-C */
+siofprintf(fp, fmt)
+	FILE *fp;
+	char *fmt;
+#endif
 #else
 siofprintf(va_alist)
 	va_dcl
@@ -207,11 +209,12 @@ siofprintf(va_alist)
 
 int
 #ifdef HAVE_STDARG_H
-sioprintf(char *fmt
-# ifdef __STDC__
-	  , ...
-# endif
-	  )
+#ifdef __STDC__
+sioprintf(char *fmt, ...)
+#else /* Non ANSI-C */
+sioprintf(fmt)
+	char *fmt;
+#endif
 #else
 sioprintf(va_alist)
 	va_dcl
