@@ -765,7 +765,7 @@ tregexec(prog, tstring)
 #ifdef	DEBUG
 	if (D_compare) {
 		fprintf(stderr,
-			"%*scomparing %s and ", 4*funclevel, "", prog->pattern);
+			"%*stcomparing '%s' and ", 4*funclevel, "", prog->pattern);
 		if (tstring != NULL) {
 			putc('\'', stderr);
 			for (s = tstring; s != NULL ; s = s->t_next)
@@ -828,7 +828,7 @@ success:
 #ifdef	DEBUG
 	if (D_matched) {
 		fprintf(stderr,
-			"%*smatched %s and '", 4*funclevel, "", prog->pattern);
+			"%*stmatched '%s' and '", 4*funclevel, "", prog->pattern);
 		for (s = tstring; s != NULL ; s = s->t_next)
 			fprintToken(stderr, s, 0);
 		fprintf(stderr, "'\n");
