@@ -14,6 +14,9 @@
  */
 
 #include "mailer.h"
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
 #include <ctype.h>
 #include <sys/file.h>
 #ifdef HAVE_FCNTL_H
@@ -411,7 +414,7 @@ iclistdbs(spl)
 			++i;
 		printf("%2d/%-2d ", i, dbip->cache_size);
 		if (dbip->ttl > 0)
-			printf("%4d ", dbip->ttl);
+			printf("%4d ", (int)dbip->ttl);
 		else
 			printf("     ");
 	}
@@ -499,11 +502,11 @@ run_db(argc, argv)
 	if (argv[3] != NULL) {
 		if (dbip->flags & DB_MAPTOLOWER) {
 			for (cp = argv[3]; *cp != '\0'; ++cp)
-				if (isascii(*cp) && isupper(*cp))
+				if (isupper(*cp))
 					*cp = tolower(*cp);
 		} else if (dbip->flags & DB_MAPTOUPPER) {
 			for (cp = argv[3]; *cp != '\0'; ++cp)
-				if (isascii(*cp) && islower(*cp))
+				if (islower(*cp))
 					*cp = toupper(*cp);
 		}
 	}
@@ -640,7 +643,7 @@ db(dbname, key)
 	if (dbip->flags & DB_MAPTOLOWER) {
 		for (cp = kbuf, realkey = key; *realkey; ++realkey) {
 		  if (cp < kbend)
-		    if (isascii(*realkey) && isupper(*realkey))
+		    if (isupper(*realkey))
 		      *cp++ = tolower(*realkey);
 		    else
 		      *cp++ = *realkey;
@@ -650,7 +653,7 @@ db(dbname, key)
 	} else if (dbip->flags & DB_MAPTOUPPER) {
 		for (cp = kbuf, realkey = key; *realkey; ++realkey) {
 		  if (cp < kbend)
-		    if (isascii(*realkey) && islower(*realkey))
+		    if (islower(*realkey))
 		      *cp++ = toupper(*realkey);
 		    else
 		      *cp++ = *realkey;
@@ -765,8 +768,7 @@ db(dbname, key)
 			 * If we got anything, it should be a byte offset into
 			 * the file named by the subtype.  Used for aliases.
 			 */
-			if (LIST(l) || !isascii(*(l->string))
-				    || !isdigit(*(l->string))) {
+			if (LIST(l) || !isdigit(*(l->string))) {
 				if (stickymem == MEM_MALLOC)
 					s_free_tree(l);
 				l = NULL;
@@ -986,7 +988,7 @@ find_domain(lookupfn, sip)
 	}
 #endif
 	/* Still failed ?  Try to look for "." */
-	sip->key = ".";
+	sip->key = (void*)".";
 	if ((l = (*lookupfn)(sip)) != NULL)
 		return l;
 	
