@@ -10,6 +10,11 @@
  */
 
 #include "mailer.h"
+#include <stdio.h>
+#include <sys/types.h>
+#ifdef HAVE_UNISTD_H
+# include <unistd.h>
+#endif
 #include <ctype.h>
 #include <fcntl.h>
 #include <sys/file.h>			/* O_RDONLY for run_praliases() */
@@ -75,6 +80,8 @@ static int run_basename ARGCV;
 static int run_recase ARGCV;
 static int run_squirrel ARGCV;
 static int run_822syntax ARGCV;
+static int run_dequote ARGCV;
+static int run_condquote ARGCV;
 
 #if	defined(XMEM) && defined(CSRIMALLOC)
 static int run_malcontents ARGCV;
@@ -132,6 +139,8 @@ struct shCmd fnctns[] = {
 {	"recase",	run_recase,		0,		0	},
 {	"squirrel",	run_squirrel,		0,		0	},
 {	"rfc822syntax",	run_822syntax,		0,		0	},
+{	"dequote",	run_dequote,		0,		0,	},
+{	"condquote",	run_condquote,		0,		0,	},
 #if	defined(XMEM) && defined(CSRIMALLOC)
 {	"malcontents",	run_malcontents,	0,		0	},
 #endif	/* CSRIMALLOC */
@@ -176,6 +185,7 @@ static struct debugind {
 	{	"bind",			&D_bind		},
 	{	"resolv",		&D_resolv	},
 	{	"memory",		&D_alloc	},
+	{	"except",		0		},
 	{	0,			0		}
 };
 
@@ -203,12 +213,16 @@ run_trace(argc, argv)
 		++argv;
 		if (strcmp(*argv, "off") == 0 || strcmp(*argv, "all") == 0) {
 			for (dbi = &buggers[0]; dbi->name != NULL; ++dbi)
+			    if (dbi->indicator)
 				*(dbi->indicator) = (**argv == (debug?'a':'o'));
 			continue;
 		} else {
 			for (dbi = &buggers[0]; dbi->name != NULL; ++dbi) {
 				if (strcmp(*argv, dbi->name) == 0) {
-					*(dbi->indicator) = debug;
+					if (dbi->indicator == NULL)
+					  debug = !debug; /* except */
+					else
+					  *(dbi->indicator) = debug;
 					break;
 				}
 			}
@@ -640,7 +654,7 @@ rd_doit(filename, dirs)
 	  if (stat(pathbuf,&stbuf) != 0) return 0; /* Failed ?  Well, skip it */
 	  if (!S_ISREG(stbuf.st_mode))   return 0; /* Not a regular file ??   */
 
-	  sprintf(buf, "%d-%d", stbuf.st_ino, router_id);
+	  sprintf(buf, "%ld-%d", (long)stbuf.st_ino, router_id);
 
 	  if (eqrename(pathbuf, buf, 1) < 0)
 	    return 0;		/* something is wrong, erename() complains.
@@ -1060,12 +1074,13 @@ run_praliases(argc, argv)
 	int c, verbose;
 	char *indexfile, buf[8192], ibuf[BUFSIZ];
 	struct siobuf *osiop = NULL;
+	int tabsep = 0;
 
 
 	verbose = 0; indexfile = NULL;
 	optind = 1;
 	errflg = 0;
-	while ((c = getopt(argc, argv, "vo:")) != EOF) {
+	while ((c = getopt(argc, argv, "vo:t")) != EOF) {
 		switch (c) {
 		case 'v':
 			++verbose;
@@ -1073,6 +1088,9 @@ run_praliases(argc, argv)
 		case 'o':
 			indexfile = optarg;
 			break;
+		case 't':
+			tabsep = 1;
+			break;
 		default:
 			++errflg;
 			break;
@@ -1113,7 +1131,7 @@ run_praliases(argc, argv)
 
 	for (h = e->e_headers; h != NULL; h = h->h_next) {
 		h->h_descriptor = &aliashdr;
-		h->h_contents = hdr_scanparse(e, h, 1);
+		h->h_contents = hdr_scanparse(e, h, 1, 0);
 		h->h_stamp = hdr_type(h);
 		if (h->h_stamp == BadHeader) {
 			hdr_errprint(e, h, stderr, "alias expansion");
@@ -1253,8 +1271,8 @@ run_listexpand(avl, il)
 {
 	struct header hs;
 	struct envelope *e;
-	struct address *ap;
-	struct token **pt, *t;
+	struct address *ap, *aroot = NULL, **atail = &aroot;
+	struct token *t;
 	struct addr *pp;
 	struct conscell *al, *alp = NULL, *tmp;
 	char *localpart, *origaddr, *attributenam;
@@ -1263,8 +1281,8 @@ run_listexpand(avl, il)
 	char *comment, *erroraddress;
 	u_char *s, *s_end;
 	int privilege = -1;
-	struct siobuf *osiop;
-	FILE *mfp, *fp, f;
+	struct siobuf *osiop = NULL;
+	FILE *mfp = NULL, *fp, f;
 	char buf[4096];
 	char DSNbuf[4096+200];
 	int fd2;
@@ -1272,6 +1290,9 @@ run_listexpand(avl, il)
 	volatile FILE *errstr = stderr;
 	char *notary = NULL;
 	int no_dsn = 0;
+	int okaddresses = 0;
+	int errcount = 0;
+	int linecnt = 0;
 
 	extern struct conscell *s_value;
 
@@ -1354,14 +1375,8 @@ run_listexpand(avl, il)
 	hs.h_descriptor = &aliashdr;
 	hs.h_pname = (char*) car(avl)->string;
 
-	/* create h->h_lines */
-	/*
-	 * It is best to maintain a line at a time as tokens, so errors will
-	 * print out nicely.
-	 */
 	initline(4096);
-	pt = &hs.h_lines;
-	t = NULL;
+
 	/*
 	 * These hoops are so we can do this for both real stdin and
 	 * the fake I/O stuff in ../libsh.
@@ -1384,6 +1399,7 @@ run_listexpand(avl, il)
 	setvbuf(fp, buf, _IOFBF, sizeof buf);
 	c = 0;
 	while ((n = getline(fp)) > 0) {
+		++linecnt;
 		/*
 		 * For sendmail compatibility, addresses may not cross line
 		 * boundaries, and line boundary is just as good an address
@@ -1420,9 +1436,81 @@ run_listexpand(avl, il)
 		/* end of sendmail compatibility kluge */
 /* #endif */
 
-		*pt = t = makeToken(linebuf, n);
+		/* create h->h_lines */
+		/*
+		 * It is best to maintain a line at a time as tokens,
+		 * so errors will print out nicely.
+		 */
+
+		hs.h_lines = t = makeToken(linebuf, n);
 		t->t_type = Line;
-		pt = &(t->t_next);
+
+		/* fix up any trailing comma (more sendmail
+		   compatibility kluges) */
+		s = t->t_pname + TOKENLEN(t)-1;
+		if (c && (*s == ',' || *s == '\n'))
+		  *s = '\0';
+
+		hs.h_contents = hdr_scanparse(e, &hs, 1, 1);
+		hs.h_stamp = hdr_type(&hs);
+
+		if (hs.h_stamp == BadHeader || hs.h_contents.a == NULL) {
+		  ++errcount;
+		  if (hs.h_stamp == BadHeader) {
+		    if (erroraddress != NULL) {
+		      if (!iserrmessage()) {
+			if (mfp == NULL) {
+			  if ((mfp = mail_open(MSG_RFC822)) != NULL) {
+			    osiop = siofds[FILENO(mfp)];
+			    siofds[FILENO(mfp)] = NULL;
+			    fprintf(mfp, "channel error\n");
+			    fprintf(mfp, "to <%s>\n", erroraddress);
+			    fprintf(mfp, "to <postmaster>\n");
+			    fprintf(mfp, "env-end\n");
+			    fprintf(mfp, "From: Error Channel <MAILER-DAEMON>\n");
+			    fprintf(mfp, "To: %s\n", erroraddress);
+			    fprintf(mfp, "Cc: The Post Office <postmaster>\n");
+			    fprintf(mfp, "Subject: Error in %s\n", comment);
+			    fprintf(mfp, "Precedence: junk\n\n");
+			    /* Print the report: */
+			    fprintf(mfp,"Input file line number %d:\n",linecnt);
+			    hdr_errprint(e, &hs, mfp, comment);
+			  }
+			} else { /* mfp != NULL */
+			  fprintf(mfp,"Input file line number %d:\n",linecnt);
+			  hdr_errprint(e, &hs, mfp, comment);
+			}
+		      }
+		      if (errcount == 1) /* At the first time only! */
+			printf("%s\n", erroraddress);
+		    }
+		    fprintf(stderr,"Input file line number %d:\n",linecnt);
+		    hdr_errprint(e, &hs, stderr, comment);
+		  } else {		/* if (hs.h_contents.a == NULL) */
+#if 0 /* Hmmm... We CAN have empty input lines! */
+		    if (errcount == 1) {
+		      /* Print only for the first intance.. */
+		      if (erroraddress != NULL)
+			printf("%s\n", erroraddress);
+		      fprintf(stderr, "listexpand: null input on line %d of STDIN\n", linecnt);
+		    }
+#endif
+		  }
+		  continue;
+		}
+
+		*atail = hs.h_contents.a;
+		while (*atail != NULL)
+		  atail = &((*atail)->a_next);
+
+		++okaddresses;
+		
+	}
+
+
+	if (mfp != NULL) {
+	  siofds[FILENO(mfp)] = osiop;
+	  mail_close(mfp);
 	}
 
 	fclose(fp);	/* Now we discard the stdio buffers, but not the
@@ -1430,60 +1518,37 @@ run_listexpand(avl, il)
 	dup2(fd2,0);	/* Return the fd to descriptor 0..		*/
 	close(fd2);	/* .. and discard the backup copy..		*/
 
-	if (t == NULL) {	/* if the file is empty, use error address */
+	if (okaddresses == 0) {	/* if the file is empty, use error address */
+
 		if (erroraddress == NULL)
 			erroraddress = "postmaster";
 
-		*pt = t = makeToken(erroraddress, strlen(erroraddress));
+		hs.h_lines = t = makeToken(erroraddress, strlen(erroraddress));
 		t->t_type = Line;
-		pt = &(t->t_next);
+
+		/* fix up any trailing comma (more sendmail
+		   compatibility kluges) */
+		s = t->t_pname + TOKENLEN(t)-1;
+		if (c && (*s == ',' || *s == '\n'))
+		  *s = '\0';
+
+		hs.h_contents = hdr_scanparse(e, &hs, 1, 1);
+		hs.h_stamp = hdr_type(&hs);
+
+		if (hs.h_stamp == BadHeader || hs.h_contents.a == NULL) {
+		  /* OUTCH!  Even the "erroraddress" parameter is illegal! */
+		  fprintf(stderr,"listexpand: input parameters bad, empty input, even 'erroraddress' bad!\n");
+		} else {
+
+		  *atail = hs.h_contents.a;
+		  while (*atail != NULL)
+		    atail = &((*atail)->a_next);
+		}
 	}
 
-	/* fix up any trailing comma (more sendmail compatibility kluges) */
-
-	s = t->t_pname + TOKENLEN(t)-1;
-	if (c && (*s == ',' || *s == '\n'))
-		*s = '\0';
-	*pt = NULL;
-	s = (u_char*) buf; /* For debugger uses far below... */
-
-	hs.h_contents = hdr_scanparse(e, &hs, 1);
-	hs.h_stamp = hdr_type(&hs);
-	if (hs.h_stamp == BadHeader || hs.h_contents.a == NULL) {
-	  if (hs.h_stamp == BadHeader) {
-	    if (erroraddress != NULL) {
-	      if (!iserrmessage()
-		  && (mfp = mail_open(MSG_RFC822)) != NULL) {
-		osiop = siofds[FILENO(mfp)];
-		siofds[FILENO(mfp)] = NULL;
-		fprintf(mfp, "channel error\n");
-		fprintf(mfp, "to %s\n", erroraddress);
-		fprintf(mfp, "env-end\n");
-		fprintf(mfp, "From: Error Channel <MAILER-DAEMON>\n");
-		fprintf(mfp, "To: %s\n", erroraddress);
-		fprintf(mfp, "Cc: The Post Office <postmaster>\n");
-		fprintf(mfp, "Subject: Error in %s\n", comment);
-		fprintf(mfp, "Precedence: junk\n\n");
-		hdr_errprint(e, &hs, mfp, comment);
-		siofds[FILENO(mfp)] = osiop;
-		mail_close(mfp);
-	      }
-	      printf("%s\n", erroraddress);
-	    }
-	    hdr_errprint(e, &hs, stderr, comment);
-	  } else {		/* if (hs.h_contents.a == NULL) */
-	    if (erroraddress != NULL)
-	      printf("%s\n", erroraddress);
-	    fprintf(stderr, "%s: null input\n", car(avl)->string);
-	  }
-	  if (errors_to != olderrors)
-	    free(errors_to);
-	  errors_to = olderrors;
-	  return NULL;
-	}
 	cnt = 0;
 	al = NULL;
-	for (ap = hs.h_contents.a; ap != NULL; ap = ap->a_next) {
+	for (ap = aroot; ap != NULL; ap = ap->a_next) {
 		char *av[5];
 		int rc;
 		int omem;
@@ -1666,16 +1731,18 @@ run_listaddrs(argc, argv)
 {
 	struct header hs;
 	struct envelope *e;
-	struct address *ap;
-	struct token **pt, *t;
+	struct address *ap, *aroot = NULL, **atail = &aroot;
+	struct token *t;
 	struct addr *pp;
 	int c, n, errflag, stuff;
 	char *comment, *erroraddress;
 	u_char *s, *s_end;
-	struct siobuf *osiop;
-	FILE *mfp, *fp, f;
+	struct siobuf *osiop = NULL;
+	FILE *mfp = NULL, *fp, f;
 	char buf[4096];
 	int fd2;
+	int okaddresses = 0;
+	int errcount = 0, linecnt = 0;
 	char *old_errorsto = errors_to;
 
 	errflag = 0;
@@ -1719,14 +1786,8 @@ run_listaddrs(argc, argv)
 	hs.h_descriptor = &aliashdr;
 	hs.h_pname = argv[0];
 
-	/* create h->h_lines */
-	/*
-	 * It is best to maintain a line at a time as tokens, so errors will
-	 * print out nicely.
-	 */
 	initline(4096);
-	pt = &hs.h_lines;
-	t = NULL;
+
 	/*
 	 * These hoops are so we can do this for both real stdin and
 	 * the fake I/O stuff in ../libsh.
@@ -1749,6 +1810,7 @@ run_listaddrs(argc, argv)
 	setvbuf(fp, buf, _IOFBF, sizeof buf);
 	c = 0;
 	while ((n = getline(fp)) > 0) {
+		++linecnt;
 		/*
 		 * For sendmail compatibility, addresses may not cross line
 		 * boundaries, and line boundary is just as good an address
@@ -1784,68 +1846,115 @@ run_listaddrs(argc, argv)
 		}
 		/* end of sendmail compatibility kluge */
 /* #endif */
+		/* create h->h_lines */
+		/*
+		 * It is best to maintain a line at a time as tokens,
+		 * so errors will print out nicely.
+		 */
 
-		*pt = t = makeToken(linebuf, n);
+		hs.h_lines = t = makeToken(linebuf, n);
 		t->t_type = Line;
-		pt = &(t->t_next);
+
+		/* fix up any trailing comma (more sendmail
+		   compatibility kluges) */
+		s = t->t_pname + TOKENLEN(t)-1;
+		if (c && (*s == ',' || *s == '\n'))
+		  *s = '\0';
+
+		hs.h_contents = hdr_scanparse(e, &hs, 1, 1);
+		hs.h_stamp = hdr_type(&hs);
+
+		if (hs.h_stamp == BadHeader || hs.h_contents.a == NULL) {
+		  ++errcount;
+		  if (hs.h_stamp == BadHeader) {
+		    if (erroraddress != NULL) {
+		      if (!iserrmessage()) {
+			if (mfp == NULL) {
+			  if ((mfp = mail_open(MSG_RFC822)) != NULL) {
+			    osiop = siofds[FILENO(mfp)];
+			    siofds[FILENO(mfp)] = NULL;
+			    fprintf(mfp, "channel error\n");
+			    fprintf(mfp, "to <%s>\n", erroraddress);
+			    fprintf(mfp, "to <postmaster>\n");
+			    fprintf(mfp, "env-end\n");
+			    fprintf(mfp, "From: Error Channel <MAILER-DAEMON>\n");
+			    fprintf(mfp, "To: %s\n", erroraddress);
+			    fprintf(mfp, "Cc: The Post Office <postmaster>\n");
+			    fprintf(mfp, "Subject: Error in %s\n", comment);
+			    fprintf(mfp, "Precedence: junk\n\n");
+			    /* Print the report: */
+			    fprintf(mfp,"Input file line number %d:\n",linecnt);
+			    hdr_errprint(e, &hs, mfp, comment);
+			  }
+			} else { /* mfp != NULL */
+			  fprintf(mfp,"Input file line number %d:\n",linecnt);
+			  hdr_errprint(e, &hs, mfp, comment);
+			}
+		      }
+		      if (errcount == 1) /* At the first time only! */
+			printf("%s\n", erroraddress);
+		    }
+		    fprintf(stderr,"Input file line number %d:\n",linecnt);
+		    hdr_errprint(e, &hs, stderr, comment);
+		  } else {		/* if (hs.h_contents.a == NULL) */
+#if 0
+		    if (errcount == 1) {
+		      /* Print only for the first intance.. */
+		      if (erroraddress != NULL)
+			printf("%s\n", erroraddress);
+		      fprintf(stderr, "%s: null input on line\n", argv[0]);
+		    }
+#endif
+		  }
+		  continue;
+		}
+
+		*atail = hs.h_contents.a;
+		while (*atail != NULL)
+		  atail = &((*atail)->a_next);
+
+		++okaddresses;
 	}
 
+	if (mfp != NULL) {
+	  siofds[FILENO(mfp)] = osiop;
+	  mail_close(mfp);
+	}
+
+
 	fclose(fp);	/* Now we discard the stdio buffers, but not the
 			   fd number 0!  Actually we use a copy of it..	*/
 	dup2(fd2,0);	/* Return the fd to descriptor 0..		*/
 	close(fd2);	/* .. and discard the backup copy..		*/
 
-	if (t == NULL) {	/* if the file is empty, use error address */
+	if (okaddresses == 0) {	/* if the file is empty, use error address */
+
 		if (erroraddress == NULL)
 			erroraddress = "postmaster";
 
-		*pt = t = makeToken(erroraddress, strlen(erroraddress));
+		hs.h_lines = t = makeToken(erroraddress, strlen(erroraddress));
 		t->t_type = Line;
-		pt = &(t->t_next);
+
+		/* fix up any trailing comma (more sendmail
+		   compatibility kluges) */
+		s = t->t_pname + TOKENLEN(t)-1;
+		if (c && (*s == ',' || *s == '\n'))
+		  *s = '\0';
+
+		hs.h_contents = hdr_scanparse(e, &hs, 1, 1);
+		hs.h_stamp = hdr_type(&hs);
+
+		if (hs.h_stamp == BadHeader || hs.h_contents.a == NULL) {
+		  /* OUTCH!  Even the "erroraddress" parameter is illegal! */
+		  fprintf(stderr,"%s: input parameters bad, empty input, even 'erroraddress' bad!\n", argv[0]);
+		} else {
+		  *atail = hs.h_contents.a;
+		  while (*atail != NULL)
+		    atail = &((*atail)->a_next);
+		}
 	}
 
-	/* fix up any trailing comma (more sendmail compatibility kluges) */
-
-	s = t->t_pname + TOKENLEN(t)-1;
-	if (c && (*s == ',' || *s == '\n'))
-		*s = '\0';
-	*pt = NULL;
-
-	hs.h_contents = hdr_scanparse(e, &hs, 1);
-	hs.h_stamp = hdr_type(&hs);
-	if (hs.h_stamp == BadHeader || hs.h_contents.a == NULL) {
-	  if (hs.h_stamp == BadHeader) {
-	    if (erroraddress != NULL) {
-	      if (!iserrmessage()
-		  && (mfp = mail_open(MSG_RFC822)) != NULL) {
-		osiop = siofds[FILENO(mfp)];
-		siofds[FILENO(mfp)] = NULL;
-		fprintf(mfp, "channel error\n");
-		fprintf(mfp, "to %s\n", erroraddress);
-		fprintf(mfp, "env-end\n");
-		fprintf(mfp, "From: Error Channel <MAILER-DAEMON>\n");
-		fprintf(mfp, "To: %s\n", erroraddress);
-		fprintf(mfp, "Cc: The Post Office <postmaster>\n");
-		fprintf(mfp, "Subject: Error in %s\n", comment);
-		fprintf(mfp, "Precedence: junk\n\n");
-		hdr_errprint(e, &hs, mfp, comment);
-		siofds[FILENO(mfp)] = osiop;
-		mail_close(mfp);
-	      }
-	      printf("%s\n", erroraddress);
-	    }
-	    hdr_errprint(e, &hs, stderr, comment);
-	  } else {		/* if (hs.h_contents.a == NULL) */
-	    if (erroraddress != NULL)
-	      printf("%s\n", erroraddress);
-	    fprintf(stderr, "%s: null input\n", argv[0]);
-	  }
-	  if (errors_to != old_errorsto)
-	    free(errors_to);
-	  errors_to = old_errorsto;
-	  return 1;
-	}
-	for (ap = hs.h_contents.a; ap != NULL; ap = ap->a_next) {
+	for (ap = aroot; ap != NULL; ap = ap->a_next) {
 		for (pp = ap->a_tokens; pp != NULL; pp = pp->p_next)
 			if (pp->p_type == anAddress)
 				break;
@@ -2281,7 +2390,7 @@ run_822syntax(argc, argv)
 	hs.h_lines = makeToken(argv[1], strlen(argv[1]));
 	hs.h_lines->t_type = Line;
 	hs.h_lines->t_next = NULL;
-	hs.h_contents = hdr_scanparse(&es, &hs, 1);
+	hs.h_contents = hdr_scanparse(&es, &hs, 1, 0);
 	hs.h_stamp = hdr_type(&hs);
 	if (hs.h_stamp == BadHeader) {
 		hdr_errprint(&es, &hs, stderr, "RFC822/976");
@@ -2290,3 +2399,77 @@ run_822syntax(argc, argv)
 		return 1;
 	return 0;
 }
+
+
+static int
+run_dequote(argc, argv)
+	int argc;
+	char *argv[];
+{
+	int len;
+	char *s = argv[1];
+
+	if (argc != 2)
+	  return 2; /* Bad bad! Missing/extra arg! */
+
+	len = strlen(s);
+
+	if ((*s == '"'  && s[len-1] == '"' ) ||
+	    (*s == '\'' && s[len-1] == '\'')) {
+	  s[len-1] = 0;
+	  ++s;
+	}
+	printf("%s\n",s);
+	return 0;
+}
+
+
+static int
+run_condquote(argc, argv)
+	int argc;
+	char *argv[];
+{
+	int len;
+	char *s = argv[1];
+	char *sp, *bs;
+
+	/* We remove quotes when they are not needed, and add them when
+	   they really are needed! */
+
+	if (argc != 2)
+	  return 2; /* Bad bad! Missing/extra arg! */
+
+	len = strlen(s);
+
+	sp = strchr(s,' '); /* White space ? */
+	if (!sp) sp = strchr(s,'\t');
+
+	bs = strchr(s,'\\'); /* Backslashes ? */
+
+	/* Quoted, and without a need for quotes */
+	if ((*s == '"'  && s[len-1] == '"' ) && !sp && !bs) {
+	  /* XXX: THIS SHOULD REALLY USE SOME SYNTAX SCANNER -- LIKE THAT ONE
+	          FOR SMTP: RFC821SCN()    */
+	  s = argv[1];
+	  s[len-1] = 0;
+	  ++s;
+	  printf("%s\n",s); /* Dequoted! */
+	  return 0;
+	}
+	if (!(*s == '"'  && s[len-1] == '"' ) && (sp || bs)) {
+	  /* We need to quote this */
+	  s = argv[1];
+	  putchar('"'); /* First quote */
+	  while (*s) {
+	    if (*s == '"' || *s == '\\')
+	      putchar('\\');
+	    putchar(*s);
+	    ++s;
+	  }
+	  printf("\"\n"); /* The last quote */
+	  return 0;
+	}
+	/* The original one.. */
+	printf("%s\n",s);
+	return 0;
+}
