@@ -50,10 +50,10 @@ struct headerinfo optional_hdrs[] = {
 { "resent-date",	DateTime,	nilUserType,	Resent		},
 { "encrypted",		Encrypted,	nilUserType,	normal		},
 { "errors-to",		AddressList,	Sender,		normal		},
-/* { "keywords",	PhraseList,	nilUserType,	normal	    }, */
 { "obsoletes",		MessageIDList,	nilUserType,	normal		},
-{ "references",		References,	nilUserType,	normal		},
 #if 0
+{ "keywords",		PhraseList,	nilUserType,	normal		},
+{ "references",		References,	nilUserType,	normal		},
 { "in-reply-to",	References,	nilUserType,	normal		},
 { "received",		Received,	nilUserType,	normal		},
 #endif
