@@ -8,16 +8,18 @@
  */
 
 #include "mailer.h"
+#include <stdio.h>
+#include <sys/types.h>
 #include <sys/param.h>
 #include <fcntl.h>
 #include <sys/file.h>
 #include <errno.h>
 #include "mail.h"
 #include "libz.h"
-#include "syslog.h"
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
+#include "syslog.h"
 
 #include "prototypes.h"
 
@@ -253,7 +255,7 @@ makeLetter(e, octothorp)
 			if (n <= !octothorp) break;
 		}
 		/* Ok, now we can proceed with the original agenda.. */
-		i = hdr_status(linebuf, linebuf, n);
+		i = hdr_status(linebuf, linebuf, n, octothorp);
 		if (i > 0) {		/* a real message header */
 			if (octothorp && linebuf[0] == '#')
 				continue;
@@ -318,17 +320,16 @@ makeLetter(e, octothorp)
 					progname, h->h_descriptor->class);
 				fwrite((char *)linebuf, sizeof (char), -i, stderr);
 				putc('\n', stderr);
-				h->h_contents = hdr_scanparse(e, h, octothorp);
+				h->h_contents = hdr_scanparse(e, h, octothorp, 0);
 				h->h_stamp = hdr_type(h);
 				return PERR_BADSUBMIT;
 				/* break; */
 			case eFrom:		/* pity to break the elegance */
-				hd = h->h_descriptor;
-				osem = hd->semantics;
-				hd->semantics = Mailbox;
-				h->h_contents = hdr_scanparse(e, h, octothorp);
+				osem = h->h_descriptor->semantics;
+				h->h_descriptor->semantics = Mailbox;
+				h->h_contents = hdr_scanparse(e, h, octothorp, 0);
 				h->h_stamp = hdr_type(h);
-				hd->semantics = osem;
+				h->h_descriptor->semantics = osem;
 				break;
 			case eEnvEnd:		/* more elegance breaks */
 				inheader = 1;
@@ -337,7 +338,7 @@ makeLetter(e, octothorp)
 				e->e_eHeaders = h->h_next;
 				break;
 			default:		/* a real envelope header */
-				h->h_contents = hdr_scanparse(e, h, octothorp);
+				h->h_contents = hdr_scanparse(e, h, octothorp, 0);
 				h->h_stamp = hdr_type(h);
 				break;
 			}
@@ -345,7 +346,7 @@ makeLetter(e, octothorp)
 			/* Expected RFC-822 header, and got something else.
 			   We play as if it was the end of the headers, and
 			   start of the body.  Keep the first faulty input
-			   around by reseeking into its start.			*/
+			   around by reseeking into its start.		    */
 			repos_getline(e->e_fp, offset(e->e_fp) - n);
 			break;
 		}
@@ -364,7 +365,7 @@ makeLetter(e, octothorp)
 		nh = h->h_next;
 		h->h_next = ph;
 		if (!octothorp && h->h_descriptor != NULL) {
-			h->h_contents = hdr_scanparse(e, h, 0);
+			h->h_contents = hdr_scanparse(e, h, 0, 0);
 			h->h_stamp = hdr_type(h);
 			if (!hdr_nilp(h))	/* excise null-valued headers */
 				ph = h;
@@ -920,7 +921,7 @@ mkSender(e, name, flag)
 	/* X: sprintf(buf, "%s <%s>", fullname, name); more or less */
 	sh->h_lines = makeToken(name, strlen((char *)name));
 	sh->h_lines->t_type = Line;
-	sh->h_contents = hdr_scanparse(e, sh, 0);
+	sh->h_contents = hdr_scanparse(e, sh, 0, 0);
 #endif
 	/* X: optimize: save this in a hashtable somewhere */
 	return sh;
@@ -930,6 +931,9 @@ mkSender(e, name, flag)
  * Construct a trace header line.
  */
 
+static const struct headerinfo	traceDesc =
+	{ "received", Received, nilUserType, normal };
+
 struct header *
 mkTrace(e)
 	struct envelope *e;
@@ -937,8 +941,6 @@ mkTrace(e)
 	register struct header	*h, *th;
 	struct addr *na;
 	struct address *ap;
-	static struct headerinfo	traceDesc =
-		{ "received", Received, nilUserType, normal };
 	
 	th = (struct header *)tmalloc(sizeof (struct header));
 	th->h_pname = "Received";
@@ -946,6 +948,8 @@ mkTrace(e)
 	th->h_stamp = newHeader;
 	th->h_next = 0;
 	th->h_contents.r = (struct received *)tmalloc(sizeof (struct received));
+	th->h_lines = NULL;
+
 	FindEnvelope(eRcvdFrom);
 	/* from */
 	if (h != NULL)
@@ -998,6 +1002,7 @@ mkTrace(e)
 	/* time */
 	th->h_contents.r->r_time = e->e_nowtime;
 	/* looks like:sprintf(buf, "from %s by %s via %s with %s id %s; %s"); */
+	th->h_contents.r->r_convert = NULL;
 	return th;
 }
 
@@ -1676,7 +1681,7 @@ sequencer(e, file)
 			tmp   = copycell(car(x));
 			cdr(tmp) = NULL;
 			sender1 = cdar(x);
-			gg = cdr(cddr(cadr(sender1))); /* recipient attrib! */
+			gg = cdr(cddr(car(sender1))); /* recipient attrib! */
 			errto = find_errto(gg);   /* Possible 'ERR' value */
 			if (!errto)
 			  errto = cddar(sender1);  /* Sender 'user' */
@@ -1970,7 +1975,7 @@ sequencer(e, file)
 				prctladdr(rcp->info, ofp,
 					  _CF_RECIPIENT, "recipient");
 				putc('\n', ofp);
-				/* XX: DSN data output ! */
+				/* DSN data output ! */
 				prdsndata(rcp->info, ofp,
 					  _CF_RCPTNOTARY, "recipient");
 				if (vfp != NULL) {
@@ -1987,7 +1992,7 @@ sequencer(e, file)
 					prctladdr(rcp->info, vfp,
 						  _CF_RECIPIENT, "recipient");
 					putc('\n', vfp);
-					/* XX: DSN data output ! */
+					/* DSN data output ! */
 					prdsndata(rcp->info, vfp,
 						  _CF_RCPTNOTARY, "recipient");
 				}
