@@ -8,7 +8,12 @@
  */
 
 #include "mailer.h"
+#include <stdio.h>
+#include <sys/types.h>
 #include <ctype.h>
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
 #include "io.h"
 #include "prototypes.h"
 
@@ -28,10 +33,10 @@ int D_rfc822 = 0; /* Debug traceing */
  */
 
 union misc
-hdr_scanparse(e, h, commentflag)
+hdr_scanparse(e, h, commentflag, no_line_crossing)
 	struct envelope *e;
 	struct header *h;
-	int commentflag;
+	int commentflag, no_line_crossing;
 {
 	register struct token *t;
 	long	len;
@@ -89,7 +94,9 @@ hdr_scanparse(e, h, commentflag)
 			ocp = cp;
 			nt = t;
 			scan_t = scan822(&cp, len, c1, c2, commentflag, &nt);
-			if (nt != t) {		/* compound token across line */
+			if (nt != t && !no_line_crossing) {
+				/* compound token across line */
+
 				while (t != nt)
 					t = t->t_next;
 				/* len should be 0 */
@@ -294,6 +301,7 @@ mkMessageId(e, unixtime)
 	h->h_descriptor = msgidDesc;
 	h->h_stamp = newHeader;
 	h->h_next = 0;
+#if 0
 	cp = rfc822tz(&unixtime, &ts, 1);
 	for (s = cp, p = tzbuf; *s != '\0'; ++s) {
 		if (isascii(*s) && isupper(*s))
@@ -311,9 +319,16 @@ mkMessageId(e, unixtime)
 		     ts->tm_year % 100, monthname[ts->tm_mon], ts->tm_mday,
 		     ts->tm_hour, ts->tm_min, ts->tm_sec,
 		     cp, e->e_file, ++genseq, myhostname);
+#else /* New way to do this .. more compressed one.. */
+	ts = gmtime(&unixtime);
+	sprintf((char *)buf, "<%04d%02d%02d%02d%02d%02dZ%s+%d@%s>",
+		ts->tm_year + 1900, ts->tm_mon + 1, ts->tm_mday,
+		ts->tm_hour,        ts->tm_min,     ts->tm_sec,
+		e->e_file,          ++genseq,       myhostname);
+#endif
 	h->h_lines = makeToken(buf, strlen((char *)buf));
 	h->h_lines->t_type = Line;
-	h->h_contents = hdr_scanparse(e, h, 0);
+	h->h_contents = hdr_scanparse(e, h, 0, 0);
 	/* OPTIMIZE: save this in a hashtable somewhere */
 	return h;
 }
@@ -340,7 +355,7 @@ mkToHeader(e, buf)
 	h->h_next = 0;
 	h->h_lines = makeToken(buf, strlen((char *)buf));
 	h->h_lines->t_type = Line;
-	h->h_contents = hdr_scanparse(e, h, 0);
+	h->h_contents = hdr_scanparse(e, h, 0, 0);
 	/* OPTIMIZE: save this in a hashtable somewhere */
 	return h;
 }
@@ -428,6 +443,15 @@ hdr_print(h, fp)
 	}
 	switch (h->h_descriptor->semantics) {
 	case Received:
+		if (h->h_lines != NULL) {
+			/* Write out the original lines if possible */
+			for (t = h->h_lines; t != NULL; t = t->t_next) {
+				fwrite((char *)t->t_pname, sizeof (char),
+				       (int)(TOKENLEN(t)), fp);
+				putc('\n', fp);
+			}
+			break;
+		}
 		if ((ap = h->h_contents.r->r_from) != NULL) {
 			fprintf(fp, " from ");
 			printAddress(fp, ap->a_tokens, 0);
