@@ -1021,13 +1021,13 @@ printaddrs(v)
 	       cp < v->cfp->contents + stbuf.st_size - 1; ++cp) {
 	    if (*cp == '\n') {
 	      *cp = '\0';
-	      if (*++cp == 's')
+	      if (*++cp == _CF_SENDER)
 		break;
 	      switch (*cp) {
-	      case 'l':
+	      case _CF_LOGIDENT:
 		v->cfp->logident = cp + 2;
 		break;
-	      case 'e':
+	      case _CF_ERRORADDR:
 		/* overload cfp->mark to be from addr*/
 		v->cfp->mark = cp+2 - v->cfp->contents;
 		break;
