@@ -20,6 +20,8 @@
 
 #include "prototypes.h"
 
+extern char *rfc822date __((time_t *));
+
 #ifndef strnsave /* unless it is a debugging macro.. */
 extern char *strnsave();
 #endif
@@ -43,7 +45,6 @@ struct not {
 	char *orcpt;
 	char *notify;
 	int   notifyflgs;
-	char *rcpt;
 	char *rcpntp;
 };
 
@@ -109,15 +110,15 @@ scnotaryreport(errfp,notary,haserrsp)
 	     by the rcpntp.. */
 	  char *s = notary->rcpntp;
 	  char *p;
-	  while (*s != ' ' && *s != '\t' && *s != 0) ++s;
+	  /* Channel name */
+	  s = skip821address(s);
 	  while (*s && (*s == ' ' || *s == '\t')) ++s;
-	  /* XX: If the 'host' part has spaces in it ?? */
-	  while (*s != ' ' && *s != '\t' && *s != 0) ++s;
+	  /* "Host" component */
+	  s = skip821address(s);
 	  while (*s && (*s == ' ' || *s == '\t')) ++s;
-	  p = strrchr(s,' ');
-	  if (!p) p = strrchr(s,'\t');
-	  if (p) *p = 0;
-	  /* Ok, we have it chopped ..  Quite a butcher job, but.. */
+	  /* "User" component */
+	  p = skip821address(s);
+	  *p = 0; /* Either it points to a NIL, or to a white-space */
 	  rcpt = s;
 	}
 
@@ -182,10 +183,9 @@ msgerror(vp, offset, message)
 	  return;
 	}
 	vp->cfp->haderror = 1;
-	fprintf(fp, "%c%c%ld:%ld:%ld:%ld\t%s\t%s\n",
+	fprintf(fp, "%c%c%ld:%ld:%ld\t%s\t%s\n",
 		_CF_DIAGNOSTIC, _CFTAG_NORMAL, offset,
 		vp->headeroffset, vp->drptoffset,
-		vp->dnsrecipient ? (vp->dnsrecipient - vp->cfp->contents) : -1,
 		notary, message);
 	fflush(fp);
 #ifdef HAVE_FSYNC
@@ -258,7 +258,6 @@ reporterrs(cfpi)
 	char *envid;
 	int notarycnt = 0;
 	int notaryspc = 0;
-	char *dnsrecipient = NULL;
 	struct ctlfile *cfp;
 	char *rcpntpointer;
 	int no_error_report = 0;
@@ -315,7 +314,7 @@ reporterrs(cfpi)
 	    continue;
 
 	  /* Line fmt:
-	     <TAGS> offset ':' headeroffset ':' drptidx <TAB> NOTARY-DATA <TAB> MESSAGE <NL> */
+	     <TAGS> offset ':' headeroffset ':' drptidx : dnsrcpnt <TAB> NOTARY-DATA <TAB> MESSAGE <NL> */
 
 	  cp = cfp->contents + *lp + 2;
 
@@ -343,14 +342,13 @@ reporterrs(cfpi)
 	      ++cp;
 	  }
 	  if (*cp == ':') {
-	    int offs = atoi(++cp);
-	    if (offs > lastoffset)
-	      offs = -1;
-	    if (offs > 0)
-	      dnsrecipient = cfp->contents + offs;
+	    /* This was a field called DNSRECIPIENT -- which
+	       was never used in actuality ... */
 	    while (isascii(*cp) && (isdigit(*cp) || *cp == '-'))
 	      ++cp;
 	  }
+	  /* If ever need to add more integer offsets, add them
+	     here with a colon prefix... */
 	  notary = NULL;
 	  if (*cp == '\t') {
 	    notary = ++cp;
@@ -376,7 +374,6 @@ reporterrs(cfpi)
 	    notaries[notarycnt].orcpt  = NULL;
 	    notaries[notarycnt].notify = NULL;
 	    notaries[notarycnt].notifyflgs = 0;
-	    notaries[notarycnt].rcpt   = dnsrecipient;
 	    notaries[notarycnt].rcpntp = rcpntpointer;
 	    if (drptidx > 0) {
 	      char *d = cfp->contents + drptidx;
@@ -455,7 +452,7 @@ reporterrs(cfpi)
 		  break;
 	      }
 	      fputs("env-end\n",errfp); /* Envelope ends */
-	      fseek(fp,(off_t)0,0);		/* Rewind! Start building the report */
+	      fseek(fp, (off_t)0, SEEK_SET); /* Rewind! Start building the report */
 
 	      if (no_error_report == 0)
 		fprintf(errfp, "To: %s\n", eaddr);
@@ -572,7 +569,7 @@ reporterrs(cfpi)
 	       use those headers on output ! */
 	    fputs(cfp->contents+headeridx, errfp);
 	    putc('\n', errfp); /* Newline in between headers and the body.. */
-	    fseek(fp, (off_t)(cfp->msgbodyoffset), 0);
+	    fseek(fp, (off_t)(cfp->msgbodyoffset), SEEK_SET);
 	  } else {
 	    /* Scan the input, and drop off the Zmailer
 	       envelope headers */
@@ -589,7 +586,7 @@ reporterrs(cfpi)
 	    else {
 	      fputs("< Eh, no content in the ORIGINAL message ???  >\n",errfp);
 	      fputs("< We will dump also transporter envelope here >\n",errfp);
-	      fseek(fp,(off_t)0,0);
+	      fseek(fp, (off_t)0, SEEK_SET);
 	    }
 	  }
 	  if (has_errors && !never_full_content &&
