@@ -180,6 +180,22 @@ int global_maxkids = 1000;
 time_t now;
 FILE *statuslog = NULL;
 
+FILE *vfp_open(cfp)
+struct ctlfile *cfp;
+{
+  FILE *vfp;
+
+  setreuid(0, cfp->uid);
+  vfp = fopen(cfp->vfpfn, "a+");
+  setreuid(0, 0);
+  if (!vfp) return NULL; /* Failure to open */
+
+  fseek(vfp, (off_t)0, SEEK_END);
+  setvbuf(vfp, (char *)NULL, _IOLBF, 0);   /* Set LINEBUFFERING */
+  return vfp;
+}
+
+
 static void cfp_free __((struct ctlfile *cfp, struct spblk *spl));
 static void cfp_free(cfp, spl)
 struct ctlfile *cfp;
@@ -197,11 +213,14 @@ struct spblk   *spl;
 
 	/* And from memory */
 
-	if (cfp->vfp) {
-	  fprintf(cfp->vfp,
-		  "ordered deletion of task file from scheduler memory (%s)\n",
-		  cfp->mid);
-	  fclose(cfp->vfp);
+	if (cfp->vfpfn) {
+	  FILE *vfp = vfp_open(cfp);
+	  if (vfp) {
+	    fprintf(vfp,
+		    "ordered deletion of task file from scheduler memory (%s)\n",
+		    cfp->mid);
+	    fclose(vfp);
+	  }
 	}
 
 	if (cfp->head != NULL) {
@@ -233,6 +252,7 @@ void free_cfp_memory(cfp)
 struct ctlfile *cfp;
 {
 	if (cfp->contents)	free(cfp->contents);
+	if (cfp->vfpfn)		free(cfp->vfpfn);
 	if (cfp->mid)		free(cfp->mid);
 	if (cfp->erroraddr)	free(cfp->erroraddr);
 	if (cfp->logident)	free(cfp->logident);
@@ -1049,16 +1069,12 @@ static struct ctlfile *schedule(fd, file, cehead, ino)
 	}
 	for (vp = cfp->head; vp != NULL; vp = vp->next[L_CTLFILE]) {
 	  vtxdo(vp, cehead, file);
-	  if (cfp->vfp)
-	    fseek(cfp->vfp, (off_t)0, 2);
 	}
-	/* If no verbose trace, don't keep contents in core!
-	   this means HUGE memory save at busy machines! */
-	if (cfp->vfp == NULL)
-	  if (cfp->contents != NULL) {
-	    free(cfp->contents);
-	    cfp->contents = NULL;
-	  }
+	/* Now we have no more need for the contents in core */
+	if (cfp->contents != NULL) {
+	  free(cfp->contents);
+	  cfp->contents = NULL;
+	}
 
 	sp_install(cfp->id, (u_char *)cfp, 0, spt_mesh[L_CTLFILE]);
 	++global_wrkcnt;
@@ -1096,6 +1112,7 @@ slurp(fd, ino)
 	contents = emalloc(stbuf.st_size+1);
 	if (eread(fd, contents, stbuf.st_size) != stbuf.st_size) { /* slurp */
 	  close(fd);
+	  free(contents); /* Failed to read ?!?! */
 	  return NULL;
 	}
 	contents[stbuf.st_size] = 0;
@@ -1109,7 +1126,7 @@ slurp(fd, ino)
 	cfp->envctime = stbuf.st_ctime;
 	cfp->contents = contents;
 	/* 
-	   cfp->vfp = NULL;
+	   cfp->vfpfn = NULL;
 	   cfp->head = NULL;
 	   cfp->mark = V_NONE;
 	   cfp->haderror = 0;
@@ -1231,6 +1248,7 @@ lockverify(cfp,cp)		/* Return 1 when lock process does not exist */
 	return 0;	/* Locking PID does exist.. */
 }
 
+
 /*
  *  The  vtxprep() does deeper analysis on jobs described at the file.
  *  It verifies possible locks (if they are still valid), and gathers
@@ -1294,10 +1312,13 @@ static struct ctlfile *vtxprep(cfp, file)
 	       * won't errorneously delete it.
 	       */
 	      close(cfp->fd);	/* Was opened on  schedule() */
-	      if (cfp->vfp != NULL) {
-		fprintf(cfp->vfp,
-			"New scheduler: Skipped a job-file because it is held locked by PID=%6.6s\n",cp+1);
-		fclose(cfp->vfp);
+	      if (cfp->vfpfn != NULL) {
+		FILE *vfp = vfp_open(cfp);
+		if (vfp) {
+		  fprintf(vfp,
+			  "New scheduler: Skipped a job-file because it is held locked by PID=%6.6s\n",cp+1);
+		  fclose(vfp);
+		}
 	      }
 	      cfp_free(cfp, NULL);
 	      ++vtxprep_skip;
@@ -1443,21 +1464,16 @@ static struct ctlfile *vtxprep(cfp, file)
 	      deletemsg(cp+2, cfp);
 	      break;
 	    case _CF_VERBOSE:
+	      cfp->vfpfn = strsave(cp+2);
 
-	      setreuid(0, cfp->uid);
-
-	      cfp->vfp = fopen(cp+2, "a+");
-
-	      setreuid(0, 0);
-
-	      if (cfp->vfp != NULL) {
-		fseek(cfp->vfp, (off_t)0, 2);
-		setvbuf(cfp->vfp, (char *)NULL,	_IOLBF, 0);
+	      {
+		FILE *vfp = vfp_open(cfp);
+		if (vfp != NULL && cfp->mid != NULL)
+		  fprintf(vfp,
+			  "scheduler processing %s\n",
+			  cfp->mid);
+		if (vfp) fclose(vfp);
 	      }
-	      if (cfp->vfp != NULL && cfp->mid != NULL)
-		fprintf(cfp->vfp,
-			"scheduler processing %s\n",
-			cfp->mid);
 	      break;
 	    case _CF_TURNME:
 	      printf("TURNME: %s\n",cp+2);
@@ -1550,10 +1566,13 @@ static struct ctlfile *vtxprep(cfp, file)
 	}
 	if (cfp->mid == NULL || cfp->logident == NULL
 	    || estat(mfpath, &stbuf) < 0) {
-	  if (cfp->vfp != NULL) {
-	    fprintf(cfp->vfp,
-		    "aborted due to missing information\n");
-	    fclose(cfp->vfp);
+	  if (cfp->vfpfn != NULL) {
+	    FILE *vfp = vfp_open(cfp);
+	    if (vfp) {
+	      fprintf(vfp,
+		      "aborted due to missing information\n");
+	      fclose(vfp);
+	    }
 	  }
 	  cfp_free(cfp, NULL);
 	  free(offarr);
