@@ -969,7 +969,7 @@ int ok, justfree;
 
 	  /* Clean vertices 'proc'-pointers,  randomize the
 	     order of thread vertices  (or sort by spool file
-	     ctime, if in AGEORDER..) */
+	     mtime, if in AGEORDER..) */
 	  thread_vertex_shuffle(thr);
 
 	  thr->attempts += 1;
@@ -1059,7 +1059,7 @@ time_t retrytime;
 	 * clamp retry time to a predictable interval so we
 	 * eventually bunch up deliveries.
 	 */
-	if (retrytime < now+63)
+	if (retrytime > 100000 && retrytime < now+63)
 	  vtx->wakeup = now;
 #if 0
 	skew = vtx->wakeup % vtx->thgrp->ce.interval;
@@ -1076,7 +1076,11 @@ time_t retrytime;
 	   as such causes simultaneous start of transporters, which
 	   causes "somewhat" spiky load behaviour */
 
-	vtx->wakeup += vtx->thgrp->ce.retries[thr->retryindex] * vtx->thgrp->ce.interval;
+	skew = vtx->thgrp->ce.retries[thr->retryindex] * vtx->thgrp->ce.interval;
+	if (retrytime <= 100000 &&
+	    (int)retrytime > skew)
+	  skew = retrytime;
+	vtx->wakeup += skew;
 #endif
 	thr->retryindex++;
 
@@ -1437,11 +1441,11 @@ FILE *fp;
 		thr->proc->thread == thr) {
 	      ++procs;
 	      fprintf(fp," P=%-5d",thr->proc->pid);
-	      fprintf(fp," HA=%ds",(now - thr->proc->hungertime));
+	      fprintf(fp," HA=%ds",(int)(now - thr->proc->hungertime));
 	      if (thr->proc->feedtime == 0)
 		fprintf(fp," FA=never");
 	      else
-		fprintf(fp," FA=%ds",(now - thr->proc->feedtime));
+		fprintf(fp," FA=%ds",(int)(now - thr->proc->feedtime));
 	      fprintf(fp," OF=%-2d", thr->proc->overfed);
 	    } else if (thr->wakeup > now)
 	      fprintf(fp," W=%ds",(int)(thr->wakeup - now));
