@@ -463,10 +463,10 @@ shutdown_kids()
 	  if (proc->pid > 0 && proc->tofd >= 0) {
 	    /* Send the death-marker to the kid, and
 	       then close the command channel */
-	    kill(SIGQUIT, proc->pid);
 	    write(proc->tofd,"\n\n",2);
 	    pipes_shutdown_child(proc->tofd);
 	    proc->tofd = -1;
+	    kill(proc->pid, SIGQUIT);
 	  }
 }
 
@@ -657,7 +657,7 @@ time_t timeout;
 	  tv.tv_sec = 0;
 	tv.tv_usec = 0;
 
-	maxf = -1;
+	maxf = 0;
 	_FD_ZERO(rdmask);
 	_FD_ZERO(wrmask);
 	readsockcnt = 0;
