@@ -48,8 +48,10 @@ extern char *procselect;
 /* dispatch table for diagnostic types */
 
 static int u_ok       __((struct vertex *, long, long, long, char*, char*));
+static int u_ok2      __((struct vertex *, long, long, long, char*, char*));
 static int u_deferred __((struct vertex *, long, long, long, char*, char*));
 static int u_error    __((struct vertex *, long, long, long, char*, char*));
+static int u_error2   __((struct vertex *, long, long, long, char*, char*));
 static int u_retryat  __((struct vertex *, long, long, long, char*, char*));
 
 static struct diagcodes {
@@ -57,8 +59,10 @@ static struct diagcodes {
 	int	(*fcn)();
 } diags[] = {
 		{	"ok",		u_ok		},
+		{	"ok2",		u_ok2		},
 		{	"deferred",	u_deferred	},
 		{	"error",	u_error		},
+		{	"error2",	u_error2	},
 		{	"retryat",	u_retryat	},
 		{	NULL,		NULL		}
 };
@@ -254,11 +258,13 @@ unctlfile(cfp, no_unlink)
 	  if (verbose)
 	    printf("   and %s/\n", path);
 
-	  if (cfp->vfp != NULL) {
-	    fseek(cfp->vfp, (off_t)0, 2);
-	    fprintf(cfp->vfp,
-		    "scheduler done processing %s\n", cfp->mid);
-	    fclose(cfp->vfp);
+	  if (cfp->vfpfn != NULL) {
+	    extern FILE *vfp_open __((struct ctlfile *));
+	    FILE *vfp = vfp_open(cfp);
+	    if (vfp) {
+	      fprintf(vfp, "scheduler done processing %s\n", cfp->mid);
+	      fclose(vfp);
+	    }
 	  }
 	} else {
 	  /* We will LOOSE this from the schedules -- add info about
@@ -655,8 +661,8 @@ expire(vp, index)
 
 	if (index < 0) {
 	  /* Expire from the LAST index to the first, this way
-	     we won't do mistake of calling indixes after they
-	     have been deleted.. */
+	     we won't do the mistake of referring indixes after
+	     they have been deleted.. */
 	  for (i = vp->ngroup -1; i >= 0; --i)
 	    expaux(vp, vp->index[i], emsg);
 	} else
@@ -666,6 +672,7 @@ expire(vp, index)
 	  free(emsg);
 }
 
+
 /*ARGSUSED*/
 static int u_ok(vp, index, inum, offset, notary, message)
 	struct vertex *vp;
@@ -673,15 +680,16 @@ static int u_ok(vp, index, inum, offset, notary, message)
 	char	*notary;
 	char	*message;
 {
+#if 0
 	/* printf("%s: %d/%d/%s/ok %s\n", vp->cfp->logident, inum, offset,
 	   notary, message ? message : "-"); */
 	if (vp->cfp->vfp != NULL && vp->cfp->contents != NULL) {
-	  fseek(vp->cfp->vfp, (off_t)0, 2);
+	  fseek(vp->cfp->vfp, (off_t)0, SEEK_END);
 	  fprintf(vp->cfp->vfp, "%s: ok %s\n",
 		  vp->cfp->contents + offset + 2 + _CFTAG_RCPTPIDSIZE,
 		  message == NULL ? "(sent)" : message);
 	}
-
+#endif
 	if (vp->notaryflg & NOT_SUCCESS) {
 	  /* Save/process info regarding delivery receipts! */
 	  msgerror(vp, offset, message);
@@ -695,6 +703,36 @@ static int u_ok(vp, index, inum, offset, notary, message)
 	return 1;
 }
 
+/*ARGSUSED*/
+static int u_ok2(vp, index, inum, offset, notary, message)
+	struct vertex *vp;
+	long	index, inum, offset;
+	char	*notary;
+	char	*message;
+{
+#if 0
+	/* printf("%s: %d/%d/%s/ok %s\n", vp->cfp->logident, inum, offset,
+	   notary, message ? message : "-"); */
+	if (vp->cfp->vfp != NULL && vp->cfp->contents != NULL) {
+	  fseek(vp->cfp->vfp, (off_t)0, SEEK_END);
+	  fprintf(vp->cfp->vfp, "%s: ok %s\n",
+		  vp->cfp->contents + offset + 2 + _CFTAG_RCPTPIDSIZE,
+		  message == NULL ? "(sent)" : message);
+	}
+#endif
+	if (vp->notaryflg & NOT_SUCCESS) {
+	  vp->cfp->haderror = 1; /* The transporter logged it for us! */
+	}
+
+	if (statuslog)
+	  logstat(statuslog,vp,"ok");
+
+	/* Delete this vertex from scheduling datasets */
+	vtxupdate(vp, index, 1);
+	return 1;
+}
+
+
 static int u_deferred(vp, index, inum, offset, notary, message)
 	struct vertex *vp;
 	long	index, inum, offset;
@@ -711,13 +749,14 @@ static int u_deferred(vp, index, inum, offset, notary, message)
 	     vp->orig[L_HOST]->name); */
 	  vp->message = strsave(message);
 	}
-
+#if 0
 	if (vp->cfp->vfp != NULL && vp->cfp->contents != NULL) {
-	  fseek(vp->cfp->vfp, (off_t)0, 2);
+	  fseek(vp->cfp->vfp, (off_t)0, SEEK_END);
 	  fprintf(vp->cfp->vfp, "%s: deferred %s\n",
 		  vp->cfp->contents + offset + 2 + _CFTAG_RCPTPIDSIZE,
 		  message == NULL ? "(unknown)" : message);
 	}
+#endif
 	/*
 	 * Even though we may get several of these per web entry,
 	 * the heuristic in reschedule() to ignore the request if
@@ -741,14 +780,14 @@ static int u_error(vp, index, inum, offset, notary, message)
 
 	if (!procselect && vp->notaryflg & NOT_FAILURE)
 	  msgerror(vp, offset, message);
-
+#if 0
 	if (vp->cfp->vfp != NULL && vp->cfp->contents != NULL) {
-	  fseek(vp->cfp->vfp, (off_t)0, 2);
+	  fseek(vp->cfp->vfp, (off_t)0, SEEK_END);
 	  fprintf(vp->cfp->vfp, "%s: error %s\n",
 		  vp->cfp->contents + offset + 2 + _CFTAG_RCPTPIDSIZE,
 		  message);
 	}
-
+#endif
 	if (statuslog)
 	  logstat(statuslog,vp,"error");
 
@@ -757,6 +796,39 @@ static int u_error(vp, index, inum, offset, notary, message)
 	return 1;
 }
 
+/* A variant where the TRANSPORT AGENT has logged the report into
+   the file! */
+static int u_error2(vp, index, inum, offset, notary, message)
+	struct vertex *vp;
+	long	index, inum, offset;
+	char	*notary;
+	char	*message;
+{
+	if (message == NULL)
+	  message = "(unknown)";
+	fprintf(stderr,
+		"%s %s: %ld/%ld/%s/error %s\n", timestring(),
+		vp->thgrp->ce.command, inum, offset, notary, message);
+
+	/* We don't need to log it! */
+	vp->cfp->haderror = 1; /* Mark it into the incore dataset */
+#if 0
+	if (vp->cfp->vfp != NULL && vp->cfp->contents != NULL) {
+	  fseek(vp->cfp->vfp, (off_t)0, SEEK_END);
+	  fprintf(vp->cfp->vfp, "%s: error %s\n",
+		  vp->cfp->contents + offset + 2 + _CFTAG_RCPTPIDSIZE,
+		  message);
+	}
+#endif
+	if (statuslog)
+	  logstat(statuslog,vp,"error");
+
+	/* Delete this vertex from scheduling datasets */
+	vtxupdate(vp, index, 0);
+	return 1;
+}
+
+
 /*
  * specify relative (w/ leading +) or absolute (w/o leading +) retry time.
  */
@@ -770,16 +842,13 @@ static int u_retryat(vp, index, inum, offset, notary, message)
 	time_t	retrytime;
 	char	*cp;
 
-	if (*message == '+') {
-	  mytime(&retrytime);
+	if (*message == '+')
 	  ++message;
-	} else
-	  retrytime = 0;
 	for (cp = message; *cp != '\0' && isdigit(*cp); ++cp)
 	  continue;
 	if (*cp != '\0')
 	  *cp++ = '\0';
-	retrytime += atoi(message);
+	retrytime = atoi(message);
 	message = cp;
 	if (*message == '\0')
 	  message = NULL;
@@ -792,13 +861,14 @@ static int u_retryat(vp, index, inum, offset, notary, message)
 	     vp->orig[L_HOST]->name); */
 	  vp->message = strsave(message);
 	}
-
+#if 0
 	if (vp->cfp->vfp != NULL && vp->cfp->contents != NULL) {
-	  fseek(vp->cfp->vfp, (off_t)0, 2);
+	  fseek(vp->cfp->vfp, (off_t)0, SEEK_END);
 	  fprintf(vp->cfp->vfp, "%s: retryat %d %s\n",
 		  vp->cfp->contents + offset + 2 + _CFTAG_RCPTPIDSIZE,
 		  (int)retrytime, message == NULL ? "(unknown)" : message);
 	}
+#endif
 	/*
 	 * Even though we may get several of these per web entry,
 	 * the heuristic in reschedule() to ignore the request if
