@@ -15,6 +15,7 @@
 #include <sys/stat.h>
 #include "zmsignal.h"
 #include "malloc.h"
+#include "syslog.h"
 
 #ifdef	HAVE_RESOLVER
 #include "netdb.h"
@@ -191,6 +192,9 @@ main(argc, argv)
 	  exit(EX_USAGE);
 	}
 
+	/* We need this latter on .. */
+	openlog("hold", LOG_PID, LOG_MAIL);
+
 	while (!getout) {
 	  char *s;
 	  /* Input:
