@@ -4,7 +4,7 @@
  */
 
 /*
- *	Copyright 1994 by Matti Aarnio
+ *	Copyright 1994-1997 by Matti Aarnio
  *
  * To really understand how headers (and their converted versions)
  * are processed you do need to draw a diagram.
@@ -170,7 +170,7 @@ ctladdr(cp)
 	/* CHANNEL: */
 	ap->channel = cp;
 	/* While not space: */
-	while (*cp != 0 && (*cp != ' ' && *cp != '\t')) ++cp;
+	cp = skip821address(cp);
 	*cp++ = '\0';
 	/* While space: */
 	while (*cp != 0 && (*cp == ' ' || *cp == '\t')) ++cp;
@@ -231,7 +231,7 @@ ctlopen(file, channel, host, exitflagp, selectaddr, matchrouter)
 	    dirprefix[0] = 0;
 
 
-	d.msgfd = -1; /* Always the zero is not good for your health .. */
+	d.msgfd = -1; /* The zero is not always good for your health .. */
 	if ((d.ctlfd = open(file, O_RDWR, 0)) < 0) {
 	  char cwd[MAXPATHLEN], buf[MAXPATHLEN+MAXPATHLEN+100];
 	  int e = errno;	/* Save it over the getwd() */
@@ -279,7 +279,7 @@ ctlopen(file, channel, host, exitflagp, selectaddr, matchrouter)
 				PROT_READ|PROT_WRITE,
 				MAP_FILE|MAP_SHARED|MAP_VARIABLE,
 				d.ctlfd, 0);
-	if ((long)d.ctlmap == -1L)
+	if ((int)d.ctlmap == -1)
 	  d.ctlmap = NULL; /* Failed ?? */
 #else
 	d.ctlmap = NULL;
@@ -412,12 +412,15 @@ ctlopen(file, channel, host, exitflagp, selectaddr, matchrouter)
 	    d.recipients = rp;
 	    rp->status = EX_OK;
 	    rp->newmsgheader = NULL;
+	    rp->drptoffset   = -1;
+	    rp->headeroffset = -1;
 	    prevrp = rp;
 	    break;
 	  case _CF_RCPTNOTARY:
 	    /*  IETF-NOTARY-DSN  DATA */
 	    ++s;
 	    if (prevrp != NULL) {
+	      prevrp->drptoffset = d.offset[i];
 	      while (*s) {
 		while (*s && (*s == ' ' || *s == '\t')) ++s;
 		if (cistrncmp("NOTIFY=",s,7)==0) {
@@ -520,6 +523,7 @@ ctlopen(file, channel, host, exitflagp, selectaddr, matchrouter)
 		   rp = rp->next) {
 		rp->newmsgheader    = &msgheaders   [headers_cnt];
 		rp->newmsgheadercvt = &msgheaderscvt[headers_cnt];
+		rp->headeroffset    = d.offset[i];
 	      }
 
 	      msgheaders   [++headers_cnt] = NULL;
@@ -617,6 +621,10 @@ ctlopen(file, channel, host, exitflagp, selectaddr, matchrouter)
 	  free(mfpath);
 #endif
 
+	/* The message file mtime -- arrival of the message to the system */
+	d.msgmtime = stbuf.st_mtime;
+
+	/* Estimate the size of the message file when sent out.. */
 	d.msgsizeestimate = stbuf.st_size + largest_headersize;
 	/* A nice fudge factor, usually this is enough..                 */
 	/* Add 3% for CRLFs.. -- assume average line length of 35 chars. */
