@@ -1,3 +1,12 @@
+/*
+ *	DNSGETRR -- module for ZMailer; common stuff for DNS using
+ *		    transport agents
+ *
+ *	A lot of changes all around over the years by Matti Aarnio
+ *	<mea@nic.funet.fi>, copyright 1992-1997
+ */
+
+
 #include "hostenv.h"
 #include <stdio.h>
 #ifdef __linux__
