@@ -1,7 +1,7 @@
 /*
  *	Copyright 1988 by Rayan S. Zachariassen, all rights reserved.
  *	This will be free software, but only when it is finished.
- *	Copyright 1992-1996 Matti Aarnio -- MIME processing et.al.
+ *	Copyright 1992-1997 Matti Aarnio -- MIME processing et.al.
  */
 
 /* History:
@@ -45,6 +45,7 @@
 #include <unistd.h> /* F_LOCK is there at some systems.. */
 #endif
 #include "mail.h"
+#include "syslog.h"
 #include "zmsignal.h"
 #include "ta.h"
 #include "splay.h"
@@ -273,19 +274,21 @@ int mmdf_mode = 0;		/* Write out MMDF-style mail folder
 long eofindex = -1;		/* When negative, putmail() can't truncate() */
 int dirhashes = 0;
 
-extern RETSIGTYPE wantout();
+extern RETSIGTYPE wantout __((int));
 extern time_t time();
 extern int optind;
 extern char *optarg;
 extern int emptyline();
-extern void prversion(), getnobody(), biff(), rbiff();
+extern void biff __((char *, char *, long));
+extern void rbiff __((struct biffer *));
+extern void prversion __((const char *)), getnobody();
 extern struct passwd *getpwnam();
 extern struct passwd *getpwuid();
-extern int setupuidgid();
-extern int createfile();
-extern int exstat();
-extern int creatembox();
-extern char *exists(), *strerror();
+extern int setupuidgid __((struct rcpt *, int, int));
+extern int createfile __((struct rcpt *, char *, uid_t));
+extern int exstat __((struct rcpt *, char *, struct stat *, int (*)() ));
+extern int creatembox __((struct rcpt *, char **, uid_t*, gid_t*, struct passwd *));
+extern char *exists __((char *, struct rcpt *));
 extern void setrootuid();
 extern void process __((struct ctldesc *dp));
 extern void deliver __((struct ctldesc *dp, struct rcpt *rp, char *timestring));
@@ -465,6 +468,9 @@ main(argc, argv)
 	} else
 	  logfp = NULL;
 
+	/* We need this latter on .. */
+	openlog("mailbox", LOG_PID, LOG_MAIL);
+
 	getnobody();
 
 	while (!getout) {
@@ -495,6 +501,9 @@ main(argc, argv)
 	  setreuid(0,0); /* We begin as roots..  process() may change us */
 	  currenteuid = 0;
 
+	  notary_setxdelay(0); /* Our initial speed estimate is
+				  overtly optimistic.. */
+
 	  dp = ctlopen(file, channel, host, &getout, NULL, NULL);
 	  if (dp == NULL) {
 	    printf("#resync %s\n",file);
@@ -831,6 +840,11 @@ deliver(dp, rp, timestring)
 	char *mboxlocks = getzenv("MBOXLOCKS");
 	char *filelocks = NULL;
 	char *locks;
+	time_t starttime, endtime;
+
+	time(&starttime);
+	notary_setxdelay(0); /* Our initial speed estimate is
+				overtly optimistic.. */
 
 	if (mboxlocks == NULL || *mboxlocks == 0 )
 	  mboxlocks = MBOXLOCKS_default;
@@ -876,7 +890,7 @@ deliver(dp, rp, timestring)
 	    }
 	    return;
 	  }
-	  if (!setupuidgid(rp, uid, 1)) {
+	  if (!setupuidgid(rp, uid, -1)) {
 	    setrootuid(rp);
 	    return;
 	  }
@@ -963,8 +977,6 @@ deliver(dp, rp, timestring)
 	    DIAGNOSTIC(rp, EX_TEMPFAIL,
 		       "System mailbox configuration is wrong!  No such directory (%s)!  Aargh!",maildirs[0]);
 
-	    openlog("scheduler", LOG_PID, LOG_MAIL);
-
 	    if (!mailbox) 
 	      syslog(LOG_ALERT, "ZMailer mailbox can't open local mail folder directory! (Using BUILTIN list of choices)\n");
 	    else
@@ -1215,13 +1227,6 @@ deliver(dp, rp, timestring)
 	
 	if (S_ISREG(st.st_mode)) {
 
-	  if (fp) {
-	    fflush(fp);
-#ifdef HAVE_FSYNC
-	    fsync(fdmail);
-#endif
-	  }
-
 	  /* Previously acquired locks need to be released,
 	     preferrably in reverse order */
 
@@ -1272,9 +1277,11 @@ deliver(dp, rp, timestring)
 	}
 
 	setrootuid(rp);
+	time(&endtime);
 
 	if (fp != NULL) {
 	  fclose(fp);		/* this closes fdmail */
+	  notary_setxdelay((int)(endtime-starttime));
 	  notaryreport(rp->addr->user,"delivery",
 		       "2.2.0 (Delivered successfully)",
 		       "x-local; 250 (Delivered successfully)");
@@ -1319,8 +1326,6 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 	  return NULL;
 	}
 
-	/* append From_ line and print out the header */
-
 	if (!mmdf_mode &&
 	    *(rp->addr->user) != TO_PIPE && eofindex > 0L) {
 
@@ -1380,6 +1385,8 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 	if (*fromuser == 0)
 	  fromuser = "MAILER-DAEMON";
 
+	/* Add the From_ line and print out the header */
+
 	if (fprintf(fp, "%s%s %s", FROM_, fromuser, timestring) < 0
 	    || writeheaders(rp,fp,"\n",convert_qp,0) < 0) {
 	  notaryreport(rp->addr->link->user, "failed",
@@ -1470,6 +1477,12 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 	    goto write_failure;
 	}
 
+	if (fflush(fp) != 0)
+	  goto write_failure;
+#ifdef HAVE_FSYNC
+	fsync(fdmail);
+#endif
+
 	/* This is actually Linux-thing, we reset the last access time
 	   to be that of before we did read the file, last modification
 	   into current moment -- At Linux 1.3.8x the system is a bit
@@ -1527,11 +1540,16 @@ program(dp, rp, timestring)
 	struct passwd *pw;
 	FILE *errfp;
 	FILE *fp;
+	time_t starttime, endtime;
+
+
+	time(&starttime);
 
 	i = 0;
 	env[i++] = "SHELL=/bin/sh";
 	env[i++] = "IFS= \t\n";
 	cp = buf;
+	*cp = 0; /* Trunc the buf string... */
 	cpe = buf + sizeof(buf);
 	if ((s = getzenv("PATH")) == NULL)
 	  env[i++] = "PATH=/usr/bin:/bin:/usr/ucb";
@@ -1689,6 +1707,8 @@ program(dp, rp, timestring)
 	} else
 		i = EX_OK;
 
+	time(&endtime);
+	notary_setxdelay((int)(endtime-starttime));
 	notaryreport(NULL,NULL,NULL,NULL); /* XX: proper values ? */
 	DIAGNOSTIC(rp, i, "%s", buf);
 	return;
