@@ -17,6 +17,7 @@
 #include <sys/stat.h>
 #include "mail.h"
 #include "malloc.h"
+#include "syslog.h"
 #include "ta.h"
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
@@ -228,6 +229,9 @@ main(argc, argv)
 	}
 	mailer = argv[optind];
 
+	/* We need this latter on .. */
+	openlog("sm", LOG_PID, LOG_MAIL);
+
 	if ((mp = readsmcf(cf, mailer)) == NULL)
 	  exit(EX_OSFILE);
 	if (mp->flags & MO_REMOTEFROM)
