@@ -50,6 +50,7 @@
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include "mail.h"
+#include "syslog.h"
 #include "ta.h"
 #include "malloc.h"
 
@@ -160,6 +161,7 @@ int debug = 0;
 int verbosity = 0;
 int conndebug = 0;
 int timeout = 0;		/* how long do we wait for response? (sec.) */
+int conntimeout = 3*60;		/* connect() timeout */
 int gotalarm =0;		/* indicate that alarm happened! */
 #ifndef	HAVE_MMAP
 int readalready = 0;		/* does buffer contain valid message data? */
@@ -233,6 +235,9 @@ typedef struct {
   char **pipecmds;
   struct rcpt **pipercpts;
   FILE *verboselog;
+  int hsize;
+  int msize;
+  int rcptcnt;
 } SmtpState;
 
 time_t now;
@@ -259,7 +264,7 @@ extern char **environ;
 extern int deliver __((SmtpState *SS, struct ctldesc *dp, struct rcpt *startrp, struct rcpt *endrp));
 extern int writebuf __((SmtpState *SS, char *buf, int len));
 extern int writemimeline __((SmtpState *SS, char *buf, int len, int convertmode, int* lastchp));
-extern int appendlet __((SmtpState *SS, struct ctldesc *dp, int hsize, int msize, int convertmode));
+extern int appendlet __((SmtpState *SS, struct ctldesc *dp, int convertmode));
 extern int smtpopen __((SmtpState *SS, char *host, int noMX));
 extern int smtpconn __((SmtpState *SS,  char *host, int noMX));
 extern int smtp_ehlo __((SmtpState *SS, char *strbuf));
@@ -296,15 +301,17 @@ extern RETSIGTYPE sig_pipe __((int));
 extern RETSIGTYPE sig_alarm __((int));
 extern int getmyhostname();
 extern void stachmyaddresses();
-extern int cistrcmp(), cistrncmp();
 extern void getdaemon();
 
-#ifdef HAVE_STDARG_H
+extern void smtpclose __((SmtpState *SS));
+
+#if defined(HAVE_STDARG_H) && defined(__STDC__)
 extern void report __((SmtpState *SS, char *fmt, ...));
 #else
-extern void report __(());
+extern void report();
 #endif
 
+static time_t starttime, endtime;
 
 static char *logtag()
 {
@@ -487,7 +494,7 @@ main(argc, argv)
 	}
 	if (errflg || optind > argc) {
 	  fprintf(stderr,
-		  "Usage: %s [-8|-8H|-7][-e][-r][-x][-E][-P][-W][-T timeout][-h myhostname][-l logfile][-p portnum][-c channel] [host]\n", argv[0]);
+		  "Usage: %s [-8|-8H|-7][-e][-r][-x][-E][-P][-W][-T timeout][-h myhostname][-l logfile][-p portnum][-c channel][-F forcedest][-L localidentity] [host]\n", argv[0]);
 	  exit(EX_USAGE);
 	}
 	if (optind < argc) {
@@ -527,6 +534,8 @@ main(argc, argv)
 	getnobody();
 	getdaemon();
 
+	/* We need this latter on .. */
+	openlog("smtp", LOG_PID, LOG_MAIL);
 
 	/* defer opening a connection until we know there is work */
 	smtpstatus = EX_DATAERR;
@@ -585,7 +594,7 @@ main(argc, argv)
 	    if (host && strcmp(s,host) != 0) {
 	      if (SS.smtpfp) {
 		smtpstatus = smtpwrite(&SS, 0, "QUIT", NULL);
-		fclose(SS.smtpfp); SS.smtpfp = NULL;
+		smtpclose(&SS);
 		notary_setwtt(NULL);
 		if (logfp) {
 		  fprintf(logfp, "%s#\t(closed SMTP channel - new host)\n", logtag());
@@ -621,6 +630,9 @@ main(argc, argv)
 	    fflush(stdout);
 	    continue;
 	  }
+
+	  time(&starttime);
+	  notary_setxdelay(0);
 	
 	  if (punthost)
 	    smtphost = punthost;
@@ -642,6 +654,8 @@ main(argc, argv)
 	    if (smtpstatus != EX_OK) {
 	      /* SMTPOPEN() made diagnostics data -- we report.. */
 	      struct rcpt *rp;
+	      time(&endtime);
+	      notary_setxdelay((int)(endtime-starttime));
 	      /* NOTARY: addres / action / status / diagnostic */
 	      for (rp = dp->recipients; rp != NULL; rp = rp->next) {
 		notaryreport(rp->addr->user,NULL,NULL,NULL);
@@ -666,7 +680,7 @@ main(argc, argv)
 	}
 	/* Close the channel -- if it is open anymore .. */
 	if (SS.smtpfp) {
-	  fclose(SS.smtpfp);
+	  smtpclose(&SS);
 	  if (logfp) {
 	    fprintf(logfp, "%s#\t(closed SMTP channel - final close)\n", logtag());
 	  }
@@ -720,7 +734,7 @@ process(SS, dp, smtpstatus, host, noMX)
 		smtpstatus = deliver(SS, dp, rphead, rp->next);
 	      /*
 		 if (openstatus != EX_OK) {
-		 fclose(SS->smtpfp); SS->smtpfp = NULL;
+		 smtpclose(SS);
 		 notary_setwtt(NULL);
 		 if (logfp)
 		 fprintf(logfp, "%s#\t(closed SMTP channel - after delivery failure)\n", logtag());
@@ -728,6 +742,8 @@ process(SS, dp, smtpstatus, host, noMX)
 	       */
 	      rphead = rp->next;
 	    } else {
+	      time(&endtime);
+	      notary_setxdelay((int)(endtime-starttime));
 	      while (rphead != rp->next) {
 		/* SMTP open -- meaning (propably) that we got reject
 		   from the remote server */
@@ -757,7 +773,7 @@ deliver(SS, dp, startrp, endrp)
 {
 	struct rcpt *rp = NULL;
 	int r = EX_TEMPFAIL;
-	int nrcpt, rcpt_cnt, hsize, size, tout;
+	int nrcpt, rcpt_cnt, size, tout;
 	int content_kind = 0, convertmode;
 	int ascii_clean = 0;
 	struct stat stbuf;
@@ -843,6 +859,13 @@ deliver(SS, dp, startrp, endrp)
 
 	smtp_flush(SS); /* Flush in every case */
 
+	/* Store estimate on how large a file it is */
+	if (fstat(dp->msgfd, &stbuf) >= 0)
+	  size = stbuf.st_size - dp->msgbodyoffset;
+	else
+	  size = -1;
+	SS->msize = size;
+
 	if (strcmp(startrp->addr->link->channel,"error")==0)
 	  sprintf(SMTPbuf, "MAIL From:<>");
 	else
@@ -885,12 +908,14 @@ deliver(SS, dp, startrp, endrp)
 			  we have some new input, do close the connection */
 	    if (has_readable(fileno(SS->smtpfp))) {
 	      /* Drain the input, and then close the channel */
-	      smtpwrite(SS, 1, NULL, NULL);
-	      fclose(SS->smtpfp); SS->smtpfp = NULL;
+	      int r2 = smtpwrite(SS, 1, NULL, NULL);
+	      smtpclose(SS);
 	      if (logfp)
 		fprintf(logfp, "%s#\t(closed SMTP channel - MAIL FROM:<> got two responces!)\n", logtag());
 	    }
 	  }
+	  time(&endtime);
+	  notary_setxdelay((int)(endtime-starttime));
 	  if (pipelining && SS->smtpfp /* might have timed out */)
 	    r = smtp_sync(SS, r);
 	  for (rp = startrp; rp && rp != endrp; rp = rp->next) {
@@ -929,6 +954,8 @@ deliver(SS, dp, startrp, endrp)
 	  /* RCPT TO:<...> -- pipelineable */
  	  r = smtpwrite(SS, 1, SMTPbuf, pipelining ? rp : NULL);
 	  if (r != EX_OK) {
+	    time(&endtime);
+	    notary_setxdelay((int)(endtime-starttime));
 	    if (pipelining && SS->smtpfp)
 	      r = smtp_sync(SS, r);
 	    /* NOTARY: addres / action / status / diagnostic / wtt */
@@ -942,6 +969,7 @@ deliver(SS, dp, startrp, endrp)
 	    rp->status = EX_OK;
 	  }
 	}
+	SS->rcptcnt = nrcpt;
 	if (nrcpt == 0) {
 	  /* all the RCPT To addresses were rejected, so reset server */
 	  if (SS->smtpfp)
@@ -961,6 +989,8 @@ deliver(SS, dp, startrp, endrp)
 	  /* In PIPELINING mode ... send "DATA" */
 	  r = smtpwrite(SS, 1, "DATA", startrp);
 	  if (r != EX_OK) { /* failure on pipes ? */
+	    time(&endtime);
+	    notary_setxdelay((int)(endtime-starttime));
 	    if (SS->smtpfp)
 	      r = smtp_sync(SS, r); /* Sync it.. */
 	    for (rp = startrp; rp && rp != endrp; rp = rp->next)
@@ -973,6 +1003,8 @@ deliver(SS, dp, startrp, endrp)
 	      smtpwrite(SS, 0, "RSET", NULL);
 	    return r;
 	  }
+	  time(&endtime);
+	  notary_setxdelay((int)(endtime-starttime));
 	  /* Now it is time to do synchronization .. */
 	  if (SS->smtpfp)
 	    r = smtp_sync(SS, EX_OK); /* Up & until "DATA".. */
@@ -994,6 +1026,8 @@ deliver(SS, dp, startrp, endrp)
 	} else {
 	  /* Non-PIPELINING sync mode */
 	  if ((r = smtpwrite(SS, 1, "DATA", NULL)) != EX_OK) {
+	    time(&endtime);
+	    notary_setxdelay((int)(endtime-starttime));
 	    for (rp = startrp; rp && rp != endrp; rp = rp->next)
 	      if (rp->status == EX_OK) {
 		/* NOTARY: addres / action / status / diagnostic / wtt */
@@ -1021,10 +1055,12 @@ deliver(SS, dp, startrp, endrp)
 	    fprintf(SS->verboselog,"%s\n",*hdrs++);
 	}
 
-	if ((hsize = writeheaders(startrp, SS->smtpfp, "\r\n",
-				  convertmode, 0)) < 0) {
+	if ((SS->hsize = writeheaders(startrp, SS->smtpfp, "\r\n",
+				      convertmode, 0)) < 0) {
 	  for (rp = startrp; rp != endrp; rp = rp->next)
 	    if (rp->status == EX_OK) {
+	      time(&endtime);
+	      notary_setxdelay((int)(endtime-starttime));
 	      /* NOTARY: addres / action / status / diagnostic / wtt */
 	      notaryreport(rp->addr->user,notaryacct(EX_TEMPFAIL,"*NOTOK*"),
 			   "5.4.2 (Message write timed out)",
@@ -1037,15 +1073,19 @@ deliver(SS, dp, startrp, endrp)
 	    smtpwrite(SS, 0, "RSET", NULL);
 	  return EX_TEMPFAIL;
 	}
+
+	/* Add the header size to the initial body size */
+	if (SS->msize >= 0)
+	  SS->msize += SS->hsize;
+	else
+	  SS->msize -= SS->hsize-1;
 	
 	/* Append the message body itself */
-	if (fstat(dp->msgfd, &stbuf) >= 0)
-	  size = hsize + stbuf.st_size - dp->msgbodyoffset;
-	else
-	  size = -hsize;
 
-	if ((r = appendlet(SS, dp, hsize, size, convertmode)) != EX_OK) {
+	if ((r = appendlet(SS, dp, convertmode)) != EX_OK) {
 	fail:
+	  time(&endtime);
+	  notary_setxdelay((int)(endtime-starttime));
 	  for (rp = startrp; rp && rp != endrp; rp = rp->next)
 	    if (rp->status == EX_OK) {
 	      notaryreport(rp->addr->user, notaryacct(r,"*NOTOK*"),
@@ -1058,7 +1098,7 @@ deliver(SS, dp, startrp, endrp)
 	   * Don't send the dot! 2/June/94 edwin@cs.toronto.edu
 	   */
 	  if (SS->smtpfp) {
-	    fclose(SS->smtpfp); SS->smtpfp = NULL;
+	    smtpclose(SS);
 	    if (logfp)
 	      fprintf(logfp, "%s#\t(closed SMTP channel - appendlet() failure)\n", logtag());
 	  }
@@ -1085,9 +1125,11 @@ deliver(SS, dp, startrp, endrp)
 	    fprintf(logfp, "%s#\t%s\n", logtag(), remotemsg);
 	  else
 	    fprintf(logfp, "%s#\t%d bytes, %d in header, %d recipients, %d secs for envelope, %d secs for body xfer\n",
-		    logtag(), size, hsize, nrcpt,
+		    logtag(), size, SS->hsize, nrcpt,
 		    body_start - env_start, body_end - body_start);
 	}
+	time(&endtime);
+	notary_setxdelay((int)(endtime-starttime));
 	for (rp = startrp; rp && rp != endrp; rp = rp->next) {
 	  if (rp->status == EX_OK) {
 	    char *reldel = "-";
@@ -1117,10 +1159,10 @@ static char let_buffer[BUFSIZ*8];
 #endif
 
 int
-appendlet(SS, dp,  hsize, msize, convertmode)
+appendlet(SS, dp, convertmode)
 	SmtpState *SS;
 	struct ctldesc *dp;
-	int hsize, msize, convertmode;
+	int convertmode;
 {
 	/* `convertmode' controls the behaviour of the message conversion:
 	     _CONVERT_NONE (0): send as is
@@ -1148,18 +1190,21 @@ appendlet(SS, dp,  hsize, msize, convertmode)
 	if (convertmode == _CONVERT_NONE &&
 	    readalready != 0) {
 	  rc = writebuf(SS, let_buffer, readalready);
-	  hsize += rc;
+	  SS->hsize += rc;
+	  if (statusreport)
+	    report(SS,"DATA %d/%d (%d%%)",
+		   SS->hsize, SS->msize, (SS->hsize*100+SS->msize/2)/SS->msize);
 	  if (rc != readalready) {
 	    if (gotalarm) {
 	      sprintf(remotemsg,"smtp; 500 (let_buffer write timeout!  DATA %d/%d [%d%%])",
-		      hsize, msize, (hsize*100+msize/2)/msize);
+		      SS->hsize, SS->msize, (SS->hsize*100+SS->msize/2)/SS->msize);
 	      /* return EX_IOERR; */
 	    }
 	    return EX_IOERR;
 	  }
 	  lastwasnl = (let_buffer[readalready-1] != '\n');
 	  if (statusreport)
-	    report(SS,"DATA %d (100%%)", readalready+hsize);
+	    report(SS,"DATA %d (100%%)", readalready+SS->hsize);
 	  if (fflush(fp) != 0)
 	    return EX_IOERR;
 	}
@@ -1186,19 +1231,20 @@ appendlet(SS, dp,  hsize, msize, convertmode)
 #endif /* !HAVE_MMAP */
 	    lastwasnl = (let_buffer[i-1] == '\n');
 	    rc = writebuf(SS, let_buffer, i);
-	    hsize += rc;
+	    SS->hsize += rc;
 	    if (statusreport)
 	      report(SS,"DATA %d/%d (%d%%)",
-		     hsize, msize, (hsize*100+msize/2)/msize);
+		     SS->hsize, SS->msize, (SS->hsize*100+SS->msize/2)/SS->msize);
 	    if (rc != i) {
 	      if (gotalarm) {
 		sprintf(remotemsg,"smtp; 500 (let_buffer write timeout!  DATA %d/%d [%d%%])",
-			hsize, msize, (hsize*100+msize/2)/msize);
+			SS->hsize, SS->msize, (SS->hsize*100+SS->msize/2)/SS->msize);
 		return EX_IOERR;
 	      }
 	      sprintf(remotemsg,
 		      "smtp; 500 (let_buffer write IO-error[1]! [%s] DATA %d/%d [%d%%])",
-		      strerror(errno), hsize, msize, (hsize*100+msize/2)/msize);
+		      strerror(errno),
+		      SS->hsize, SS->msize, (SS->hsize*100+SS->msize/2)/SS->msize);
 	      return EX_IOERR;
 	    }
 #ifndef HAVE_MMAP
@@ -1251,14 +1297,12 @@ appendlet(SS, dp,  hsize, msize, convertmode)
 
 	    /* Ok, write the line -- decoding QP can alter the "lastwasnl" */
 	    rc = writemimeline(SS, let_buffer, i, convertmode, &lastch);
-	    hsize += rc;
-	    if (statusreport)
-	      report(SS,"DATA %d/%d (%d%%)",
-		     hsize, msize, (hsize*100+msize/2)/msize);
+	    SS->hsize += rc;
 	    if (rc != i) {
 	      sprintf(remotemsg,
 		      "500 (let_buffer write IO-error[2]! [%s] DATA %d/%d [%d%%])",
-		      strerror(errno), hsize, msize, (hsize*100+msize/2)/msize);
+		      strerror(errno),
+		      SS->hsize, SS->msize, (SS->hsize*100+SS->msize/2)/SS->msize);
 	      MFPCLOSE
 	      return EX_IOERR;
 	    }
@@ -1281,12 +1325,13 @@ appendlet(SS, dp,  hsize, msize, convertmode)
 	  if (writebuf(SS, "\n", 1) != 1) {
 	    if (gotalarm) {
 	      sprintf(remotemsg,"500 (let_buffer write timeout!  DATA %d/%d [%d%%])",
-		      hsize, msize, (hsize*100+msize/2)/msize);
+		      SS->hsize, SS->msize, (SS->hsize*100+SS->msize/2)/SS->msize);
 	      return EX_IOERR;
 	    }
 	    sprintf(remotemsg,
 		    "500 (let_buffer write IO-error[3]! [%s] DATA %d/%d [%d%%])",
-		    strerror(errno), hsize, msize, (hsize*100+msize/2)/msize);
+		    strerror(errno),
+		    SS->hsize, SS->msize, (SS->hsize*100+SS->msize/2)/SS->msize);
 #ifndef	HAVE_MMAP
 	    if (bufferfull > 1) readalready = 0;
 #endif
@@ -1341,12 +1386,19 @@ writebuf(SS, buf, len)
 	    alarmcnt = ALARM_BLOCKSIZE;
 	    fflush(fp);
 	    alarm(timeout);	/* re-arm it */
+
+	    if (statusreport)
+	      report(SS,"DATA %d/%d (%d%%)",
+		     len-n+SS->hsize, SS->msize,
+		     ((len-n+SS->hsize)*100+SS->msize/2)/SS->msize);
 	  }
 
 	  if (state && c != '\n') {
 	    state = 0;
 	    if (c == '.') {
 	      if (putc(c, fp) == EOF || putc(c, fp) == EOF) {
+		time(&endtime);
+		notary_setxdelay((int)(endtime-starttime));
 		notaryreport(NULL,FAILED,"5.4.2 (body write error, 1)",
 			     "smtp; 500 (body write error, 1)");
 		strcpy(remotemsg, "write error 1");
@@ -1355,6 +1407,8 @@ writebuf(SS, buf, len)
 	      VLFPRINTF((SS->verboselog,".."));
 	    } else {
 	      if (putc(c, fp) == EOF) {
+		time(&endtime);
+		notary_setxdelay((int)(endtime-starttime));
 		notaryreport(NULL,FAILED,"5.4.2 (body write error, 2)",
 			     "smtp; 500 (body write error, 2)");
 		strcpy(remotemsg, "write error 2");
@@ -1364,6 +1418,8 @@ writebuf(SS, buf, len)
 	    }
 	  } else if (c == '\n') {
 	    if (putc('\r', fp) == EOF || putc(c, fp) == EOF) {
+	      time(&endtime);
+	      notary_setxdelay((int)(endtime-starttime));
 	      notaryreport(NULL,FAILED,"5.4.2 (body write error, 3)",
 			   "smtp; 500 (body write error, 3)");
 	      strcpy(remotemsg, "write error 3");
@@ -1373,6 +1429,8 @@ writebuf(SS, buf, len)
 	    state = 1;
 	  } else {
 	    if (putc(c, fp) == EOF) {
+	      time(&endtime);
+	      notary_setxdelay((int)(endtime-starttime));
 	      notaryreport(NULL,FAILED,"5.4.2 (body write error, 4)",
 			   "smtp; 500 (body write error, 4)");
 	      strcpy(remotemsg, "write error 4");
@@ -1432,6 +1490,11 @@ writemimeline(SS, buf, len, convertmode, lastcharp)
 	    alarmcnt = ALARM_BLOCKSIZE;
 	    fflush(fp);
 	    alarm(timeout);
+
+	    if (statusreport)
+	      report(SS,"DATA %d/%d (%d%%)",
+		     len-n+SS->hsize, SS->msize,
+		     ((len-n+SS->hsize)*100+SS->msize/2)/SS->msize);
 	  }
 
 	  if (convertmode == _CONVERT_8BIT) {
@@ -1501,6 +1564,8 @@ writemimeline(SS, buf, len, convertmode, lastcharp)
 
 	  if (column == 0 && c == '.') {
 	    if (putc(c, fp) == EOF) {
+	      time(&endtime);
+	      notary_setxdelay((int)(endtime-starttime));
 	      notaryreport(NULL,FAILED,"5.4.2 (body write error, 5)",
 			   "smtp; 500 (body write error, 5)");
 	      strcpy(remotemsg, "write error 5");
@@ -1511,6 +1576,8 @@ writemimeline(SS, buf, len, convertmode, lastcharp)
 
 	  if (c == '\n') {
 	    if (putc('\r', fp) == EOF) {
+	      time(&endtime);
+	      notary_setxdelay((int)(endtime-starttime));
 	      notaryreport(NULL,FAILED,"5.4.2 (body write error, 6)",
 			   "smtp; 500 (body write error, 6)");
 	      strcpy(remotemsg, "write error 6");
@@ -1520,8 +1587,10 @@ writemimeline(SS, buf, len, convertmode, lastcharp)
 	    column = -1;
 	  }
 	  if (putc(c, fp) == EOF) {
-	      notaryreport(NULL,FAILED,"5.4.2 (body write error, 7)",
-			   "smtp; 500 (body write error, 7)");
+	    time(&endtime);
+	    notary_setxdelay((int)(endtime-starttime));
+	    notaryreport(NULL,FAILED,"5.4.2 (body write error, 7)",
+			 "smtp; 500 (body write error, 7)");
 	    strcpy(remotemsg, "write error 7");
 	    return EOF;
 	  }
@@ -1631,7 +1700,7 @@ smtpopen(SS, host, noMX)
 	    sprintf(SMTPbuf, "HELO %s", myhostname);
 	    i = smtpwrite(SS, 1, SMTPbuf, NULL);
 	    if (i == EX_TEMPFAIL && SS->smtpfp) {
-	      fclose(SS->smtpfp); SS->smtpfp = NULL;
+	      smtpclose(SS);
 	      if (logfp)
 		fprintf(logfp, "%s#\t(closed SMTP channel - HELO failed ?)\n", logtag());
 	    }
@@ -1641,7 +1710,7 @@ smtpopen(SS, host, noMX)
 		return i;
 	      i = smtpwrite(SS, 1, SMTPbuf, NULL);
 	      if (i == EX_TEMPFAIL && SS->smtpfp) {
-		fclose(SS->smtpfp); SS->smtpfp = NULL;
+		smtpclose(SS);
 		if (logfp)
 		  fprintf(logfp, "%s#\t(closed SMTP channel - HELO failed(2))\n", logtag());
 	      }
@@ -1689,6 +1758,8 @@ smtpconn(SS, host, noMX)
 	  ipaddr.s_addr = inet_addr(buf);
 	  if (ipaddr.s_addr == 0xFFFFFFFF) {
 	    sprintf(remotemsg, "smtp; 500 (bad IP address: %s)", host);
+	    time(&endtime);
+	    notary_setxdelay((int)(endtime-starttime));
 	    notaryreport(NULL,FAILED,"5.1.2 (bad literal IP address)",
 			 remotemsg);
 	    if (SS->verboselog)
@@ -1725,6 +1796,8 @@ smtpconn(SS, host, noMX)
 #endif /* BIND */
 		) {
 	      sprintf(remotemsg,"smtp; 566 (gethostbyname(%s): try later)",host);
+	      time(&endtime);
+	      notary_setxdelay((int)(endtime-starttime));
 	      notaryreport(NULL,FAILED,"5.4.3 (dns lookup timeout)",remotemsg);
 	      if (SS->verboselog)
 		fprintf(SS->verboselog,"%s\n",remotemsg+6);
@@ -1740,6 +1813,8 @@ smtpconn(SS, host, noMX)
 	      host = hbuf;
 	    } else if (h_errno == TRY_AGAIN) {
 	      sprintf(remotemsg, "smtp; 466 (Canonicalize, try later: %s)", host);
+	      time(&endtime);
+	      notary_setxdelay((int)(endtime-starttime));
 	      notaryreport(NULL,FAILED,"5.4.3 (dns lookup 'try again')",remotemsg);
 	      if (SS->verboselog)
 		fprintf(SS->verboselog,"%s\n",remotemsg+6);
@@ -1749,6 +1824,8 @@ smtpconn(SS, host, noMX)
 		 even though CNAME lookups fail (see .UK) */
 #else  /* !BIND */
 	    sprintf(remotemsg, "smtp; 466 (object lookup failed on %s)", host);
+	    time(&endtime);
+	    notary_setxdelay((int)(endtime-starttime));
 	    notaryreport(NULL,FAILED,"5.4.3 (unspecified dns error)", remotemsg);
 	    if (SS->verboselog)
 	      fprintf(SS->verboselog,"%s\n", remotemsg+6);
@@ -1800,6 +1877,8 @@ smtpconn(SS, host, noMX)
 #endif /* BIND */
 		  ) {
 		sprintf(remotemsg,"smtp; 566 (gethostbyname<%s>: try later)",host);
+		time(&endtime);
+		notary_setxdelay((int)(endtime-starttime));
 		notaryreport(NULL,FAILED,"5.4.3 (dns lookup 'try again')", remotemsg);
 		if (SS->verboselog)
 		  fprintf(SS->verboselog,"%s\n",remotemsg+6);
@@ -1814,6 +1893,8 @@ smtpconn(SS, host, noMX)
 		sprintf(remotemsg,
 			"smtp; 500 (nameserver data inconsistency. No MX, no address: '%s', errno=%s)",
 			host,strerror(errno));
+	      time(&endtime);
+	      notary_setxdelay((int)(endtime-starttime));
 	      notaryreport(NULL,FAILED,"5.4.4 (nameserver data inconsistency)",
 			   remotemsg);
 	      if (SS->verboselog)
@@ -1894,6 +1975,8 @@ makeconn(SS, hp, fdp)
 	    getrr(hostbuf, sizeof hostbuf, (u_short)T_WKS, 2, SS->verboselog) != 1) {
 	  sprintf(remotemsg,"smtp; 550 (WKS checks: no SMTP capability for host %s)",
 		  hostbuf);
+	  time(&endtime);
+	  notary_setxdelay((int)(endtime-starttime));
 	  notaryreport(NULL,FAILED,"5.4.4 (WKS Checks: no SMTP capability)", remotemsg);
 	  if (SS->verboselog)
 	    fprintf(SS->verboselog,"%s\n",remotemsg+6);
@@ -1916,6 +1999,8 @@ makeconn(SS, hp, fdp)
 	for (hp_init(hp); *hp_getaddr() != NULL; hp_nextaddr()) {
 	  int i;
 	  if ((i = matchmyaddress(*hp_getaddr(),hp->h_length))) {
+	    time(&endtime);
+	    notary_setxdelay((int)(endtime-starttime));
 	    if (i == 2) {
 	      notaryreport(NULL,FAILED,"5.4.6 (trying to talk to loopback (=myself)!)","smtp; 500 (Trying to talk to loopback (=myself)!)");
 	    } else {
@@ -1927,12 +2012,13 @@ makeconn(SS, hp, fdp)
 
 	  if (SS->smtpfp) {
 	    /* Clean (close) these fds -- they have noted to leak.. */
-	    fclose(SS->smtpfp); SS->smtpfp = NULL;
+	    smtpclose(SS);
 	    if (logfp)
 	      fprintf(logfp,"%s#\t(closed SMTP channel at makeconn())\n",logtag());
 	  }
 
-	  switch (vcsetup(SS,*hp_getaddr(), hp->h_length, fdp, hostbuf)) {
+	  i = vcsetup(SS,*hp_getaddr(), hp->h_length, fdp, hostbuf);
+	  switch (i) {
 	  case EX_OK:
 	      if ((SS->smtpfp = fdopen(*fdp, "w")) == NULL) {
 		int err;
@@ -1995,6 +2081,8 @@ vcsetup(SS,ipaddrcp, ipaddrlen, fdp, hostname)
 	if ((s = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
 	  se = strerror(errno);
 	  sprintf(remotemsg, "smtp; 500 (Internal error, socket: %s)", se);
+	  time(&endtime);
+	  notary_setxdelay((int)(endtime-starttime));
 	  notaryreport(NULL,FAILED,"5.4.0 (internal error)",remotemsg);
 	  if (conndebug)
 	    fprintf(stderr, "%s\n", remotemsg+6);
@@ -2029,6 +2117,8 @@ vcsetup(SS,ipaddrcp, ipaddrlen, fdp, hostname)
 	    if (errno != EADDRINUSE && errno != EADDRNOTAVAIL) {
 	      char *s = strerror(errno);
 	      sprintf(remotemsg, "smtp; 500 (Internal error, bind: %s)", s);
+	      time(&endtime);
+	      notary_setxdelay((int)(endtime-starttime));
 	      notaryreport(NULL,FAILED,"5.4.0 (internal error, bind)",remotemsg);
 	      if (SS->verboselog)
 		fprintf(SS->verboselog,"%s\n", remotemsg+6);
@@ -2041,6 +2131,8 @@ vcsetup(SS,ipaddrcp, ipaddrlen, fdp, hostname)
 	  }
 	  if (p < (u_short)(IPPORT_RESERVED/2)) {
 	    sprintf(remotemsg, "too many busy ports");
+	    time(&endtime);
+	    notary_setxdelay((int)(endtime-starttime));
 	    notaryreport(NULL,FAILED,"5.4.0 (internal error, too many busy ports)","smtp; 500 (Internal error, too many busy ports)");
 	    if (conndebug)
 	      fprintf(stderr, "%s\n", remotemsg+6);
@@ -2070,10 +2162,15 @@ vcsetup(SS,ipaddrcp, ipaddrlen, fdp, hostname)
 	  fprintf(SS->verboselog, "Connecting to %s [%s] port %d\n",
 		  hostname, ipaddress, ntohs(sad.sin_port));
 
+
+	alarm(conntimeout);
+	gotalarm = 0;
+
 	if (connect(s, (struct sockaddr *)&sad, sizeof sad) >= 0) {
 	  int on = 1;
 /* setreuid(0,0); */
 	  *fdp = s;
+	  alarm(0);
 #if	defined(__svr4__) || defined(BSD) && (BSD-0) >= 43
 	  setsockopt(s, SOL_SOCKET, SO_KEEPALIVE, (void*)&on, sizeof on);
 #else  /* BSD < 43 */
@@ -2085,12 +2182,22 @@ vcsetup(SS,ipaddrcp, ipaddrlen, fdp, hostname)
 	}
 /* setreuid(0,0); */
 
+	if (errno == EINTR && gotalarm)
+	  errno = ETIMEDOUT; /* We use ALARM to limit the timeout.
+				At some systems the tcp connection setup
+				timeout is RATHER LONG ... */
+
 	errnosave = errno;
-	se = strerror(errno);
+	se = strerror(errnosave);
+
+	alarm(0);
+
 	sprintf(remotemsg, "smtp; 500 (connect to %s [%s]: %s)",
 		hostname, ipaddress, se);
 	if (statusreport)
 	  report(SS,"%s",remotemsg+4);
+	time(&endtime);
+	notary_setxdelay((int)(endtime-starttime));
 	notaryreport(NULL,FAILED,"5.4.1 (TCP/IP-connection failure)", remotemsg);
 
 	if (conndebug)
@@ -2123,6 +2230,7 @@ vcsetup(SS,ipaddrcp, ipaddrlen, fdp, hostname)
 	return EX_UNAVAILABLE;
 }
 
+
 RETSIGTYPE
 sig_pipe(sig)
 int sig;
@@ -2141,8 +2249,14 @@ int sig;
 }
 
 #ifdef HAVE_STDARG_H
+#ifdef __STDC__
 void
 rmsgappend(char *fmt, ...)
+#else /* Not ANSI-C */
+void
+rmsgappend(fmt)
+	char *fmt;
+#endif
 #else
 void
 rmsgappend(va_alist)
@@ -2183,6 +2297,15 @@ rmsgappend(va_alist)
  */
 
 
+void
+smtpclose(SS)
+SmtpState *SS;
+{
+	if (SS->smtpfp != NULL)
+		fclose(SS->smtpfp);
+	SS->smtpfp = NULL;
+}
+
 void
 smtp_flush(SS)
 	SmtpState *SS;
@@ -2413,6 +2536,7 @@ int smtp_sync(SS,r)
 	char *p;
 
 	alarm(timeout);
+	gotalarm = 0;
 	fflush(SS->smtpfp);			/* Flush output */
 	SS->smtp_outcount = 0;
 	SS->block_written = 0;
@@ -2545,6 +2669,8 @@ int smtp_sync(SS,r)
 	    rc = code_to_status(code, &status);
 
 	    /* Diagnose the errors, we report successes AFTER the DATA phase.. */
+	    time(&endtime);
+	    notary_setxdelay((int)(endtime-starttime));
 	    notarystatsave(SS,s,status);
 	    notaryreport(SS->pipercpts[idx]->addr->user,FAILED,NULL,NULL);
 	    diagnostic(SS->pipercpts[idx],rc, 0, "%s", remotemsg);
@@ -2706,26 +2832,32 @@ smtpwrite(SS, saverpt, strbuf, syncrp)
 	  if (err) {
 	    if (gotalarm) {
 	      strcpy(remotemsg, "Timeout on cmd write");
+	      time(&endtime);
+	      notary_setxdelay((int)(endtime-starttime));
 	      notaryreport(NULL,FAILED,"5.4.2 (timeout on cmd write)",
 			   "smtp; 500 (timeout on cmd write)");
 	    } else
 	      {
 		se = strerror(errno);
 		sprintf(remotemsg, "smtp; 500 (write to server error: %s)", se);
+		time(&endtime);
+		notary_setxdelay((int)(endtime-starttime));
 		notaryreport(NULL,FAILED,"5.4.2 (write to server, err)",remotemsg);
 	      }
 	    if (SS->verboselog)
 	      fprintf(SS->verboselog,"%s\n",remotemsg);
-	    fclose(SS->smtpfp); SS->smtpfp = NULL;
+	    smtpclose(SS);
 	    if (logfp)
 	      fprintf(logfp, "%s#\t(closed SMTP channel - timeout on smtpwrite())\n", logtag());
 	    return EX_TEMPFAIL;
 	  } else if (r != len) {
 	    sprintf(remotemsg, "smtp; 500 (SMTP cmd write failure: Only wrote %d of %d bytes!)", r, len);
+	    time(&endtime);
+	    notary_setxdelay((int)(endtime-starttime));
 	    notaryreport(NULL,FAILED,"5.4.2 (SMTP cmd partial write failure)",remotemsg);
 	    if (SS->verboselog)
 	      fprintf(SS->verboselog,"%s\n",remotemsg);
-	    fclose(SS->smtpfp); SS->smtpfp = NULL;
+	    smtpclose(SS);
 	    if (logfp)
 	      fprintf(logfp, "%s#\t(closed SMTP channel - second timeout on smtpwrite() )\n", logtag());
 	    return EX_TEMPFAIL;
@@ -2819,6 +2951,8 @@ smtpwrite(SS, saverpt, strbuf, syncrp)
 		sprintf(remotemsg,
 			"smtp; 466 (Timeout on SMTP responce read, Cmd: %s)",
 			strbuf);
+	      time(&endtime);
+	      notary_setxdelay((int)(endtime-starttime));
 	      notaryreport(NULL,FAILED,"5.4.2 (smtp transaction read timeout)",remotemsg);
 	    } else {
 		se = strerror(errno);
@@ -2830,13 +2964,15 @@ smtpwrite(SS, saverpt, strbuf, syncrp)
 		  sprintf(remotemsg,
 			  "smtp; 500 (Error on SMTP responce read: %s, Cmd: %s)",
 			  se, strbuf);
+		time(&endtime);
+		notary_setxdelay((int)(endtime-starttime));
 		notaryreport(NULL,FAILED,"5.4.2 (smtp transaction read timeout)",remotemsg);
 	      }
 	    dflag = 0;
 	    firstmx = 0;
 	    if (SS->verboselog)
 	      fprintf(SS->verboselog,"%s\n",remotemsg);
-	    fclose(SS->smtpfp); SS->smtpfp = NULL;
+	    smtpclose(SS);
 	    if (logfp)
 	      fprintf(logfp, "%s#\t(closed SMTP channel - bad responce on smtpwrite() )\n", logtag());
 	    return EX_TEMPFAIL;
@@ -2844,11 +2980,13 @@ smtpwrite(SS, saverpt, strbuf, syncrp)
 	    /* read() returned 0 .. usually meaning EOF .. */
 	    sprintf(remotemsg, "smtp; 500 (Server hung up on us! Cmd: %s)",
 		    strbuf == NULL ? "(null cmd)" : strbuf);
+	    time(&endtime);
+	    notary_setxdelay((int)(endtime-starttime));
 	    notaryreport(NULL,FAILED,"5.4.2 (server hung-up on us)",remotemsg);
 	    dflag = 0;
 	    if (SS->verboselog)
 	      fprintf(SS->verboselog,"%s\n",remotemsg);
-	    fclose(SS->smtpfp); SS->smtpfp = NULL;
+	    smtpclose(SS);
 	    if (logfp)
 	      fprintf(logfp, "%s#\t(closed SMTP channel - hangup on smtpwrite() )\n", logtag());
 	    return EX_TEMPFAIL;
@@ -2860,11 +2998,13 @@ smtpwrite(SS, saverpt, strbuf, syncrp)
 
 	if (cp >= buf+sizeof buf) {
 	  strcpy(remotemsg,"smtp; 500 (SMTP Response overran input buffer!)");
+	  time(&endtime);
+	  notary_setxdelay((int)(endtime-starttime));
 	  notaryreport(NULL,"X-BUG","5.5.0 (SMTP-responce overran input buffer!)",remotemsg);
 	  dflag = 0;
 	  if (SS->verboselog)
 	    fprintf(SS->verboselog,"%s\n",remotemsg);
-	  fclose(SS->smtpfp); SS->smtpfp = NULL;
+	  smtpclose(SS);
 	  if (logfp)
 	    fprintf(logfp, "%s#\t(closed SMTP channel - responce overrun on smtpwrite() )\n", logtag());
 	  return EX_TEMPFAIL;
@@ -2872,11 +3012,13 @@ smtpwrite(SS, saverpt, strbuf, syncrp)
 	*--cp = '\0';	/* kill the LF */
 	if (cp - buf < 5) {
 	  sprintf(remotemsg, "smtp; 500 (SMTP response '%s' unexpected!)", buf);
+	  time(&endtime);
+	  notary_setxdelay((int)(endtime-starttime));
 	  notaryreport(NULL,"X-BUG","5.5.0 (SMTP responce unexpected)",remotemsg);
 	  dflag = 0;
 	  if (SS->verboselog)
 	    fprintf(SS->verboselog,"%s\n",remotemsg);
-	  fclose(SS->smtpfp); SS->smtpfp = NULL;
+	  smtpclose(SS);
 	  if (logfp)
 	    fprintf(logfp, "%s#\t(closed SMTP channel - unexpected responce on smtpwrite() )\n", logtag());
 	  return EX_TEMPFAIL;
@@ -2935,7 +3077,13 @@ smtp_ehlo(SS, strbuf)
  */
 
 #ifdef HAVE_STDARG_H
+#ifdef __STDC__
 void report(SmtpState *SS, char *fmt, ...)
+#else /* Not ANSI-C */
+void report(SS, fmt)
+	SmtpState *SS;
+	char *fmt;
+#endif
 #else
 /* VARARGS */
 void
@@ -3010,6 +3158,8 @@ getmxrr(SS, host, mx, maxmx, depth, mxcntp)
 
 	if (depth > 3) {
 	  sprintf(remotemsg,"smtp; 500 (DNS: Recursive CNAME on '%s')",host);
+	  time(&endtime);
+	  notary_setxdelay((int)(endtime-starttime));
 	  notaryreport(NULL,FAILED,"5.4.3 (Recursive DNS CNAME)",remotemsg);
 	  fprintf(stderr, "%s\n", remotemsg);
 	  return EX_NOHOST;
@@ -3022,6 +3172,8 @@ getmxrr(SS, host, mx, maxmx, depth, mxcntp)
 	  fprintf(stderr, "res_mkquery failed\n");
 	  sprintf(remotemsg,
 		  "smtp; 466 (Internal: res_mkquery failed on host: %s)",host);
+	  time(&endtime);
+	  notary_setxdelay((int)(endtime-starttime));
 	  notaryreport(NULL,FAILED,"5.4.3 (DNS-failure)",remotemsg);
 	  return EX_SOFTWARE;
 	}
@@ -3029,6 +3181,8 @@ getmxrr(SS, host, mx, maxmx, depth, mxcntp)
 	if (n < 0) {
 	  sprintf(remotemsg,
 		  "smtp; 466 (No DNS response for host: %s)",host);
+	  time(&endtime);
+	  notary_setxdelay((int)(endtime-starttime));
 	  notaryreport(NULL,FAILED,"5.4.3 (DNS-failure)",remotemsg);
 	  return EX_TEMPFAIL;
 	}
@@ -3048,10 +3202,14 @@ getmxrr(SS, host, mx, maxmx, depth, mxcntp)
 	     * change to return EX_TEMPFAIL iff hp->aa == 0.
 	     */
 	    sprintf(remotemsg, "smtp; 500 (DNS: no such domain: %s)", host);
+	    time(&endtime);
+	    notary_setxdelay((int)(endtime-starttime));
 	    notaryreport(NULL,FAILED,"5.4.4 (DNS lookup report)",remotemsg);
 	    return EX_NOHOST;
 	  case SERVFAIL:
 	    sprintf(remotemsg, "smtp; 500 (DNS: server failure: %s)", host);
+	    time(&endtime);
+	    notary_setxdelay((int)(endtime-starttime));
 	    notaryreport(NULL,FAILED,"5.4.4 (DNS lookup report)",remotemsg);
 	    return EX_TEMPFAIL;
 #ifdef OLDJEEVES
@@ -3073,10 +3231,14 @@ getmxrr(SS, host, mx, maxmx, depth, mxcntp)
 	  case NOTIMP:
 	  case REFUSED:
 	    sprintf(remotemsg, "smtp; 500 (DNS: unsupported query: %s)", host);
+	    time(&endtime);
+	    notary_setxdelay((int)(endtime-starttime));
 	    notaryreport(NULL,FAILED,"5.4.4 (DNS lookup report)",remotemsg);
 	    return EX_NOPERM;
 	  }
 	  sprintf(remotemsg, "smtp; 500 (DNS: unknown error, MX info unavailable: %s)", host);
+	  time(&endtime);
+	  notary_setxdelay((int)(endtime-starttime));
 	  notaryreport(NULL,FAILED,"5.4.4 (DNS lookup report)",remotemsg);
 	  return EX_UNAVAILABLE;
 	}
@@ -3195,6 +3357,8 @@ getmxrr(SS, host, mx, maxmx, depth, mxcntp)
 	}
 	if (n == 0) {/* MX's exist, but their WKS's show no TCP smtp service */
 	  sprintf(remotemsg, "smtp; 500 (DNS: MX host does not support SMTP: %s)", host);
+	  time(&endtime);
+	  notary_setxdelay((int)(endtime-starttime));
 	  notaryreport(NULL,FAILED,"5.4.4 (DNS lookup report)",remotemsg);
 	  return EX_UNAVAILABLE;
 	}
