@@ -27,19 +27,23 @@
 
 #define PROG "makendbm"
 
+int aliasinput = 0;
+
 #include <errno.h>
 extern int errno;	/* Not all systems declare it in <errno.h> ... */
 
-extern char *strchr();
+/* extern char *strchr(); */
 
 extern void create_dbase __((FILE *, void *, int));
 
+extern char *skip821address __((char *));
+
 void
 usage(prog,err,errno)
 char *prog, *err;
 int errno;
 {
-  fprintf(stderr,"Usage: %s dbtype database.name [infilename|-]\n",prog);
+  fprintf(stderr,"Usage: %s [-a] dbtype database.name [infilename|-]\n",prog);
   fprintf(stderr,"  where supported dbtypes are:");
 #ifdef HAVE_NDBM_H
   fprintf(stderr," ndbm");
@@ -61,6 +65,9 @@ int errno;
 #ifdef HAVE_DB_H
   fprintf(stderr,"  (BTREE appends .db, to the actual db file name..)\n");
 #endif
+  fprintf(stderr,"\n");
+  fprintf(stderr,"  The '-a' option is for parsing input that comes in\n");
+  fprintf(stderr,"  'aliases' format:  'key: data,in,single,long,line'\n");
   fprintf(stderr," Error now: %s",err);
   fprintf(stderr,", errno=%d (%s)",errno,strerror(errno));
   fprintf(stderr,"\n");
@@ -68,13 +75,14 @@ int errno;
 }
 
 
+char linebuf[64*1024]; /* 64 kB for SINGLE line is rather much :-) */
+
 void
 create_dbase(infile,dbf,typ)
      FILE *infile;
      void *dbf;
      int typ;
 {
-	char linebuf[2048];
 	char *s, *t;
 #ifdef HAVE_NDBM_H
 	DBM *ndbmfile = dbf;
@@ -101,16 +109,27 @@ create_dbase(infile,dbf,typ)
 	  while (*t == '\t' || *t == ' ') ++t;
 
 	  if (*t == 0) continue; /* Blank line! */
+	  if (*t == '#') continue; /* Comment.. */
 
-	  s = t;
-	  while (*s && *s != '\t' && *s != ' ') ++s;
-	  if (*s) {
-	    *s = 0; /* Found end of input token, put 0 there, and look if
-		       there is some data on this line! */
-	    ++s;
-	    while (*s && (*s == '\t' || *s == ' ')) ++s;
-	    /* Point to begin of data after separating white space! */
+	  if (aliasinput) {
+	    s = skip821address(t);
+	    /* We are now at the white-space -- possibly the last
+	       char of the line prefix is a double-colon (:) */
+	    if (s > t) {
+	      if (s[-1] == ':')
+		s[-1] = 0;
+	      else if (s[0] != 0)
+		*s++ = 0; /* Zero-terminate the key, scan for input */
+	    }
+	  } else {
+	    s = t;
+	    /* Scan over the non-LWSP text */
+	    while (*s && *s != '\t' && *s != ' ') ++s;
+	    /* Stopped without line-end NIL ?  Trunc, and advance! */
+	    if (*s) *s++ = 0;
 	  }
+	  /* Scan forward, if we have some LWSP here, and then more data ? */
+	  while (*s && (*s == '\t' || *s == ' ')) ++s;
 
 	  rc = -2;
 #ifdef HAVE_NDBM_H
@@ -161,23 +180,34 @@ char *argv[];
 	FILE *infile = NULL;
 	int typ = 0;
 #ifdef HAVE_NDBM_H
-	DBM *ndbmfile;
+	DBM *ndbmfile = NULL;
 #endif
 #ifdef HAVE_GDBM_H
-	GDBM_FILE gdbmfile;
+	GDBM_FILE gdbmfile = NULL;
 #endif
 #ifdef HAVE_DB_H
-	DB *dbfile;
+	DB *dbfile = NULL;
 #endif
-	char *dbtype;
-	void *dbf;
+	char *dbtype = NULL;
+	void *dbf = NULL;
+	char *argv0 = argv[0];
+
+	aliasinput = 0;
+
+	if (argc > 3) {
+	  if (strcmp(argv[1],"-a") == 0) {
+	    aliasinput = 1;
+	    ++argv;
+	    --argc;
+	  }
+	}
 
 	/* Usage: */
 	/*  makendbm dbtype database.name [infilename|-] */
 	/* argv[] 0    1          2           3		 */
 
-	if (argc < 3) usage(argv[0],"too few arguments",0);
-	if (argc > 4) usage(argv[0],"too many arguments",0);
+	if (argc < 3) usage(argv0,"too few arguments",0);
+	if (argc > 4) usage(argv0,"too many arguments",0);
 	dbasename = argv[2];
 
 	if (argc == 4) {
@@ -189,7 +219,7 @@ char *argv[];
 	  infile = (FILE*)fopen(argv[2],"r");
 	dbtype = argv[1];
 
-	if (infile == NULL) usage(argv[0],"bad infile",errno);
+	if (infile == NULL) usage(argv0,"bad infile",errno);
 
 	typ = 0;
 #ifdef HAVE_NDBM_H
@@ -207,7 +237,7 @@ char *argv[];
 	      typ = 3;
 #endif
 	if (typ == 0)
-	  usage(argv[0],"unknown dbtype",0);
+	  usage(argv0,"unknown dbtype",0);
 
 #ifdef HAVE_NDBM_H
 	if (typ == 1) {
@@ -234,7 +264,7 @@ char *argv[];
 	}
 #endif
 	if (dbf == NULL)
-	  usage(argv[0],"Can't open dbase file",errno);
+	  usage(argv0,"Can't open dbase file",errno);
 
 	create_dbase(infile,dbf,typ);
 
